/*
 * (c) Copyright 2009-2010 Freescale Semiconductor
 *
 * (c) 2007 Pengutronix, Sascha Hauer <s.hauer@pengutronix.de>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


#include <common.h>
#include <asm/io.h>
#include <asm/errno.h>
#include <asm/arch/mx25.h>
#include <asm/arch/mx25-regs.h>
#include <asm/arch/mx25_pins.h>
#include <asm/arch/iomux.h>
#include <asm/arch/gpio.h>
#include <imx_spi.h>

#ifdef CONFIG_CMD_MMC
#include <mmc.h>
#include <fsl_esdhc.h>
#endif

#ifndef XMK_STR
# define XMK_STR(x)	#x
#endif	/* XMK_STR */

#ifndef MK_STR
# define MK_STR(x)	XMK_STR(x)
#endif	/* MK_STR */

static void led_light(iomux_pin_name_t pin, u8 val)
{
	mxc_set_gpio_dataout(pin, (val & 1));
}
#define LED_LIGHT(pin, val)	led_light(pin, val)

#ifdef CONFIG_JHRWL3_DEBUG
static void led_light_wait(iomux_pin_name_t pin)
{
	led_light(pin, 1);
	udelay(1 * 1000 * 1000);
	led_light(pin, 0);
	udelay(2 * 1000 * 1000);
}
#define LED_LIGHT_WAIT(pin)	led_light_wait(pin)
#else
#define LED_LIGHT_WAIT(pin)
#endif

DECLARE_GLOBAL_DATA_PTR;

static u32 system_rev;

u32 get_board_rev(void)
{
	return system_rev;
}

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Setup Revision
 */
static inline void setup_soc_rev(void)
{
	int reg;
	reg = __REG(IIM_BASE + IIM_SREV);
	if (!reg) {
		reg = __REG(ROMPATCH_REV);
		reg <<= 4;
	} else
		reg += CHIP_REV_1_0;
	system_rev = 0x25000 + (reg & 0xFF);
}

inline int is_soc_rev(int rev)
{
	return (system_rev & 0xFF) - rev;
}

#if 0
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * IOMUX Configuration
 */
/*----------------------------------------------------------------------
 * CLKO
 */
static inline void iomux_clko_configure(void)
{
#define CLKO_MASK	~(0x7FF << 20)	/* EN=30 DIV=29-24 */
#define CLKO_EN_	(0x1 << 30)
#define CLKO_SEL_AHB	(0x3 << 20)
#define CLKO_SEL_IPG	(0x4 << 20)	/* Select ahb_clk */
	u32	mcr;
	u32	div, ahb_clk;

	/* i.MX257 Special PIN's */
	mxc_request_iomux(MX25_PIN_CLKO, MUX_CONFIG_FUNC);

	mxc_iomux_set_pad(MX25_PIN_CLKO, PAD_CTL_DRV_HIGH);	/* DSE=High */

	/* div=32 is "4.15625MHz (=133MHz/32) */
	ahb_clk = mxc_get_clock(MXC_AHB_CLK);
	div = ((ahb_clk / 4156250) + 1) - 1;

	/* CLKO Enable */
	mcr = (__REG(CCM_MCR) & CLKO_MASK);
	mcr |= (CLKO_EN_ | (div << 24) | CLKO_SEL_AHB);
	__REG(CCM_MCR) = (u32)mcr;
}
#endif

/*----------------------------------------------------------------------
 * LCD
 */
static inline void iomux_lcd_configure(void)
{
#ifdef CONFIG_JHRWL3_LCD
	mxc_request_iomux(MX25_PIN_D14, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_D15, MUX_CONFIG_ALT5);

	mxc_iomux_set_pad(MX25_PIN_D14,  PAD_CTL_PKE_ENABLE | 
			PAD_CTL_PUE_KEEPER | PAD_CTL_100K_PD | PAD_CTL_SRE_FAST);
	mxc_iomux_set_pad(MX25_PIN_D15, PAD_CTL_PKE_ENABLE | 
			PAD_CTL_PUE_KEEPER | PAD_CTL_100K_PD | PAD_CTL_SRE_FAST);

	mxc_set_gpio_direction_output(MX25_PIN_D14, 1);
	mxc_set_gpio_direction_output(MX25_PIN_D15, 1);

	/* LCDC Reset */
	mxc_set_gpio_dataout(MX25_PIN_D15, 0);
	mxc_set_gpio_dataout(MX25_PIN_D14, 0);

	mxc_request_iomux(MX25_PIN_LD0, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD1, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD2, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD3, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD4, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD5, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD6, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD7, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD8, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD9, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD10, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD11, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD12, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD13, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD14, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LD15, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_GPIO_E, MUX_CONFIG_ALT2);
	mxc_request_iomux(MX25_PIN_GPIO_F, MUX_CONFIG_ALT2);
	mxc_request_iomux(MX25_PIN_HSYNC, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_VSYNC, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_LSCLK, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_OE_ACD, MUX_CONFIG_FUNC);

#define LCD_PAD_CTL (PAD_CTL_PKE_ENABLE | PAD_CTL_PUE_PUD | PAD_CTL_100K_PU)

	mxc_iomux_set_pad(MX25_PIN_LD0, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD1, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD2, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD3, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD4, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD5, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD6, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD7, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD8, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD9, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD10, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD11, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD12, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD13, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD14, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LD15, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_GPIO_E, MUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX25_PIN_GPIO_F, MUX_CONFIG_ALT2);
	mxc_iomux_set_pad(MX25_PIN_HSYNC, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_VSYNC, LCD_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_LSCLK, LCD_PAD_CTL | PAD_CTL_SRE_FAST);
	mxc_iomux_set_pad(MX25_PIN_OE_ACD, LCD_PAD_CTL);

#undef LCD_PAD_CTL
#else
	/* Not Configuration */
#endif
}

/*----------------------------------------------------------------------
 * LCD Backlight
 */
static inline void iomux_lcdbl_configure(void)
{
#ifdef CONFIG_JHRWL3_LCD
	mxc_request_iomux(MX25_PIN_CONTRAST, MUX_CONFIG_ALT4);
	mxc_request_iomux(MX25_PIN_PWM, MUX_CONFIG_ALT5);

#define LCDBL_CTL_CTL	(PAD_CTL_PKE_ENABLE | PAD_CTL_PUE_PUD | PAD_CTL_100K_PD)

	mxc_iomux_set_pad(MX25_PIN_CONTRAST, LCDBL_CTL_CTL);
	mxc_iomux_set_pad(MX25_PIN_PWM, LCDBL_CTL_CTL);

	mxc_set_gpio_direction_output(MX25_PIN_PWM, 0);

#undef LCDBL_CTL_CTL
#else
	/* Not Configuration */
#endif
}

/*----------------------------------------------------------------------
 * Switch Configuration
 */
static inline void iomux_switch_configure(void)
{
#ifdef CONFIG_HW_TESTPROG
	/* mode set */
	mxc_request_iomux(MX25_PIN_D9, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_D10, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_D11, MUX_CONFIG_ALT5);

#define SWITCH_PAD_CTL	(PAD_CTL_PKE_ENABLE | PAD_CTL_PUE_KEEPER | \
			PAD_CTL_100K_PU | PAD_CTL_SRE_FAST)

	/* pad control */
	mxc_iomux_set_pad(MX25_PIN_D9, SWITCH_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_D10, SWITCH_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_D11, SWITCH_PAD_CTL);

	/* input setting */
	mxc_set_gpio_direction_input(MX25_PIN_D9);
	mxc_set_gpio_direction_input(MX25_PIN_D10);
	mxc_set_gpio_direction_input(MX25_PIN_D11);

#undef SWITCH_PAD_CTL
#endif	/* CONFIG_HW_TESTPROG */
}
/*----------------------------------------------------------------------
 * LED Configuration
 */
//static inline void iomux_led_configure(void)
void iomux_led_configure(void)
{
#if defined(CONFIG_HW_TESTPROG) | defined(CONFIG_JHRWL3_DEBUG)
	/* mode set */
	mxc_request_iomux(MX25_PIN_CSI_D2, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_CSI_D3, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_CSI_D4, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_CSI_D5, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_CSI_D6, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_CSI_D7, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_CSI_D8, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_CSI_VSYNC, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_CSI_HSYNC, MUX_CONFIG_ALT5);

	/* pad control */
#define LED_PAD_CTRL1	(PAD_CTL_PKE_ENABLE | PAD_CTL_PUE_KEEPER | PAD_CTL_SRE_FAST)
#define LED_PAD_CTRL2	(PAD_CTL_PKE_ENABLE | PAD_CTL_PUE_KEEPER | PAD_CTL_SRE_SLOW)

	mxc_iomux_set_pad(MX25_PIN_CSI_D2, LED_PAD_CTRL1);
	mxc_iomux_set_pad(MX25_PIN_CSI_D3, LED_PAD_CTRL2);
	mxc_iomux_set_pad(MX25_PIN_CSI_D4, LED_PAD_CTRL1);
	mxc_iomux_set_pad(MX25_PIN_CSI_D5, LED_PAD_CTRL2);
	mxc_iomux_set_pad(MX25_PIN_CSI_D6, LED_PAD_CTRL2);
	mxc_iomux_set_pad(MX25_PIN_CSI_D7, LED_PAD_CTRL2);
	mxc_iomux_set_pad(MX25_PIN_CSI_D8, LED_PAD_CTRL2);
	mxc_iomux_set_pad(MX25_PIN_CSI_VSYNC, LED_PAD_CTRL2);
	mxc_iomux_set_pad(MX25_PIN_CSI_HSYNC, LED_PAD_CTRL2);

	/* output setting - initialize low setting */
	mxc_set_gpio_direction_output(MX25_PIN_CSI_D2, 0);
	mxc_set_gpio_direction_output(MX25_PIN_CSI_D3, 0);
	mxc_set_gpio_direction_output(MX25_PIN_CSI_D4, 0);
	mxc_set_gpio_direction_output(MX25_PIN_CSI_D5, 0);
	mxc_set_gpio_direction_output(MX25_PIN_CSI_D6, 0);
	mxc_set_gpio_direction_output(MX25_PIN_CSI_D7, 0);
	mxc_set_gpio_direction_output(MX25_PIN_CSI_D8, 0);
	mxc_set_gpio_direction_output(MX25_PIN_CSI_VSYNC, 0);
	mxc_set_gpio_direction_output(MX25_PIN_CSI_HSYNC, 0);

#undef LED_PAD_CTRL1
#undef LED_PAT_CTRL2
#endif	/* CONFIG_HW_TESTPROG | CONFIG_JHRWL3_DEBUG */
}

/*----------------------------------------------------------------------
 * Ethernet 
 */
static inline void iomux_ethphy_configure(void)
{

#define FEC_PAD_CTL1 (PAD_CTL_HYS_SCHMITZ | PAD_CTL_PUE_PUD | \
			PAD_CTL_PKE_ENABLE)
#define FEC_PAD_CTL2 (PAD_CTL_PUE_PUD)

#ifdef CONFIG_JHRWL3_0TP_BOARD
	/* mode set */
	mxc_request_iomux(MX25_PIN_FEC_MDC, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_MDIO, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_TDATA0, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_TDATA1, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_TX_EN, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_RDATA0, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_RDATA1, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_RX_DV, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_TX_CLK, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_UART2_CTS, MUX_CONFIG_ALT2);
	mxc_request_iomux(MX25_PIN_D12, MUX_CONFIG_ALT5);

	/* Input Select */
	mxc_iomux_set_input(MUX_IN_FEC_FEC_RX_ER, INPUT_CTL_PATH2);

	/* pad control */
	mxc_iomux_set_pad(MX25_PIN_FEC_MDC, FEC_PAD_CTL2);
	mxc_iomux_set_pad(MX25_PIN_FEC_MDIO, FEC_PAD_CTL1 | PAD_CTL_22K_PU);
	mxc_iomux_set_pad(MX25_PIN_FEC_TDATA0, FEC_PAD_CTL2);
	mxc_iomux_set_pad(MX25_PIN_FEC_TDATA1, FEC_PAD_CTL2);
	mxc_iomux_set_pad(MX25_PIN_FEC_TX_EN, FEC_PAD_CTL2);
	mxc_iomux_set_pad(MX25_PIN_FEC_RDATA0, FEC_PAD_CTL1);
	mxc_iomux_set_pad(MX25_PIN_FEC_RDATA1, FEC_PAD_CTL1);
	mxc_iomux_set_pad(MX25_PIN_FEC_RX_DV, FEC_PAD_CTL1);
	mxc_iomux_set_pad(MX25_PIN_FEC_TX_CLK, FEC_PAD_CTL1);
	mxc_iomux_set_pad(MX25_PIN_UART2_CTS, PAD_CTL_PUE_PUD | PAD_CTL_100K_PU);
	mxc_iomux_set_pad(MX25_PIN_D12, 0);

	/* Setup FEC Reset */
	mxc_set_gpio_direction_output(MX25_PIN_D12, 0);
#else
	/* MX25 3Stack FEC Control */
	/* setup pins for FEC */
	mxc_request_iomux(MX25_PIN_FEC_TX_CLK, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_RX_DV, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_RDATA0, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_TDATA0, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_TX_EN, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_MDC, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_MDIO, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_RDATA1, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_FEC_TDATA1, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_POWER_FAIL, MUX_CONFIG_FUNC); /* PHY INT */

	mxc_iomux_set_pad(MX25_PIN_FEC_TX_CLK, FEC_PAD_CTL1);
	mxc_iomux_set_pad(MX25_PIN_FEC_RX_DV, FEC_PAD_CTL1);
	mxc_iomux_set_pad(MX25_PIN_FEC_RDATA0, FEC_PAD_CTL1);
	mxc_iomux_set_pad(MX25_PIN_FEC_TDATA0, FEC_PAD_CTL2);
	mxc_iomux_set_pad(MX25_PIN_FEC_TX_EN, FEC_PAD_CTL2);
	mxc_iomux_set_pad(MX25_PIN_FEC_MDC, FEC_PAD_CTL2);
	mxc_iomux_set_pad(MX25_PIN_FEC_MDIO, FEC_PAD_CTL1 | PAD_CTL_22K_PU);
	mxc_iomux_set_pad(MX25_PIN_FEC_RDATA1, FEC_PAD_CTL1);
	mxc_iomux_set_pad(MX25_PIN_FEC_TDATA1, FEC_PAD_CTL2);
	mxc_iomux_set_pad(MX25_PIN_POWER_FAIL, FEC_PAD_CTL1);

	/*
	 * Set up the FEC_RESET_B and FEC_ENABLE GPIO pins.
	 * Assert FEC_RESET_B, then power up the PHY by asserting
	 * FEC_ENABLE, at the same time lifting FEC_RESET_B.
	 *
	 * FEC_RESET_B: gpio2[3] is ALT 5 mode of pin D12
	 * FEC_ENABLE_B: gpio4[8] is ALT 5 mode of pin A17
	 */
	mxc_request_iomux(MX25_PIN_A17, MUX_CONFIG_ALT5); /* FEC_EN */
	mxc_request_iomux(MX25_PIN_D12, MUX_CONFIG_ALT5); /* FEC_RESET_B */

	mxc_iomux_set_pad(MX25_PIN_A17, PAD_CTL_ODE_OpenDrain);
	mxc_iomux_set_pad(MX25_PIN_D12, 0);

	mxc_set_gpio_direction(MX25_PIN_A17, 0); /* FEC_EN */
	mxc_set_gpio_direction(MX25_PIN_D12, 0); /* FEC_RESET_B */

	/* drop PHY power */
	mxc_set_gpio_dataout(MX25_PIN_A17, 0);	/* FEC_EN */

	/* assert reset */
	mxc_set_gpio_dataout(MX25_PIN_D12, 0);	/* FEC_RESET_B */
	udelay(2);		/* spec says 1us min */

	/* turn on PHY power and lift reset */
	mxc_set_gpio_dataout(MX25_PIN_A17, 1);	/* FEC_EN */
	mxc_set_gpio_dataout(MX25_PIN_D12, 1);	/* FEC_RESET_B */
#endif	/* CONFIG_JHRWL3_0TP_BOARD */
}

/*----------------------------------------------------------------------
 * EEPROM / PMIC / RTC
 */
static inline void iomux_i2c_configure(void)
{
	int retry = 256;

#define I2C1_CLK_PAD_CTL	(0xA8)

	/* 2011.12.26
	 * i2c dummy reciver
	 */
	mxc_request_iomux(MX25_PIN_I2C1_CLK, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_I2C1_DAT, MUX_CONFIG_ALT5);

	mxc_iomux_set_pad(MX25_PIN_I2C1_CLK, I2C1_CLK_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_I2C1_DAT, I2C1_CLK_PAD_CTL);

	/* GPIO Setting */
	mxc_set_gpio_direction_output(MX25_PIN_I2C1_CLK, 1);	/* SCL output */
	mxc_set_gpio_direction_input(MX25_PIN_I2C1_DAT);	/* SDA input */

	while (retry--) {
		if (mxc_gpio_get(MX25_PIN_I2C1_DAT) == 1)
			break;
		mxc_set_gpio_dataout(MX25_PIN_I2C1_CLK, 0);

		if (mxc_gpio_get(MX25_PIN_I2C1_DAT) == 1)
			break;
		mxc_set_gpio_dataout(MX25_PIN_I2C1_CLK, 1);
	}

	/* mode set */
	mxc_request_iomux(MX25_PIN_I2C1_CLK, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_I2C1_DAT, MUX_CONFIG_FUNC);

	/* pad control */
	mxc_iomux_set_pad(MX25_PIN_I2C1_CLK, I2C1_CLK_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_I2C1_DAT, I2C1_CLK_PAD_CTL);

	/* eeprom write protect */
	mxc_request_iomux(MX25_PIN_UART1_CTS, MUX_CONFIG_ALT5);
	mxc_iomux_set_pad(MX25_PIN_UART1_CTS, PAD_CTL_PUE_PUD | PAD_CTL_100K_PU);
	mxc_set_gpio_direction_output(MX25_PIN_UART1_CTS, 1);
#undef I2C1_CLK_PAD_CTL
}

/*----------------------------------------------------------------------
 * WLAN
 */
static inline void iomux_wlan_configure(void)
{
#ifdef CONFIG_JHRWL3_0TP_BOARD
#define SDHC_PAD_CTL (PAD_CTL_PKE_ENABLE | PAD_CTL_PUE_PUD | \
		      PAD_CTL_47K_PU | PAD_CTL_SRE_FAST)

	/* SDHC setting */
	mxc_request_iomux(MX25_PIN_SD1_CMD,
			  MUX_CONFIG_FUNC | MUX_CONFIG_SION);
	mxc_request_iomux(MX25_PIN_SD1_CLK,
			  MUX_CONFIG_FUNC | MUX_CONFIG_SION);
	mxc_request_iomux(MX25_PIN_SD1_DATA0, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_SD1_DATA1, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_SD1_DATA2, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_SD1_DATA3, MUX_CONFIG_FUNC);

	mxc_iomux_set_pad(MX25_PIN_SD1_CMD, SDHC_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_SD1_CLK, SDHC_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_SD1_DATA0, SDHC_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_SD1_DATA1, SDHC_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_SD1_DATA2, SDHC_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_SD1_DATA3, SDHC_PAD_CTL);

	/* WLAN Setting */
	mxc_request_iomux(MX25_PIN_CSI_D9, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_CSI_PIXCLK, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_GPIO_C, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_GPIO_D, MUX_CONFIG_FUNC);

	mxc_iomux_set_pad(MX25_PIN_CSI_D9, PAD_CTL_PKE_ENABLE | 
				PAD_CTL_PUE_KEEPER | PAD_CTL_100K_PU);
	mxc_iomux_set_pad(MX25_PIN_CSI_PIXCLK, 
				PAD_CTL_PKE_ENABLE | PAD_CTL_PUE_KEEPER);
	mxc_iomux_set_pad(MX25_PIN_GPIO_C, 
				PAD_CTL_PKE_ENABLE | PAD_CTL_PUE_PUD);
	mxc_iomux_set_pad(MX25_PIN_GPIO_D, PAD_CTL_PKE_ENABLE |
				PAD_CTL_PUE_PUD | PAD_CTL_100K_PD);

	mxc_set_gpio_direction_output(MX25_PIN_CSI_D9, 1);
	mxc_set_gpio_direction_input(MX25_PIN_CSI_PIXCLK);
	/* Power Down Resetting */
	mxc_set_gpio_direction_output(MX25_PIN_GPIO_C, 0);	/* PDn */
	mxc_set_gpio_direction_output(MX25_PIN_GPIO_D, 0);	/* RESETn */
#undef SDHC_PAD_CTL
#endif	/* CONFIG_JHRWL3_0TP_BOARD */
}

/*----------------------------------------------------------------------
 * Speech
 */
static inline void iomux_speech_configure(void)
{
#ifdef CONFIG_HW_TESTPROG
	/* mode set */
	mxc_request_iomux(MX25_PIN_CSPI1_SCLK, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_CSPI1_MOSI, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_CSPI1_MISO, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_CSPI1_SS0, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_CSPI1_SS1, MUX_CONFIG_ALT5);	/* GPIO1_GPIO[17] */
	mxc_request_iomux(MX25_PIN_CSPI1_RDY, MUX_CONFIG_ALT5);	/* GPIO2_GPIO[22] */
	mxc_request_iomux(MX25_PIN_UART1_RTS, MUX_CONFIG_ALT5);	/* GPIO4_GPIO[24] */
	mxc_request_iomux(MX25_PIN_UART2_RTS, MUX_CONFIG_ALT5);	/* GPIO4_GPIO[24] */

#define SPI_PAD_CTL1	(PAD_CTL_HYS_SCHMITZ | PAD_CTL_PKE_ENABLE \
				| PAD_CTL_100K_PU)
#define SPI_PAD_CTL2	(PAD_CTL_PKE_ENABLE | PAD_CTL_PUE_PUD \
				| PAD_CTL_100K_PU)

	/* pad control */
	mxc_iomux_set_pad(MX25_PIN_CSPI1_SCLK, SPI_PAD_CTL1);
	mxc_iomux_set_pad(MX25_PIN_CSPI1_MOSI, SPI_PAD_CTL1);
	mxc_iomux_set_pad(MX25_PIN_CSPI1_MISO, SPI_PAD_CTL1);
	mxc_iomux_set_pad(MX25_PIN_CSPI1_SS0, SPI_PAD_CTL1);
	mxc_iomux_set_pad(MX25_PIN_CSPI1_SS1, SPI_PAD_CTL1);
	mxc_iomux_set_pad(MX25_PIN_CSPI1_RDY, SPI_PAD_CTL1);
	mxc_iomux_set_pad(MX25_PIN_UART1_RTS, SPI_PAD_CTL2);
	mxc_iomux_set_pad(MX25_PIN_UART2_RTS, SPI_PAD_CTL2 | PAD_CTL_SRE_FAST);

	/* gpio setting */
	mxc_set_gpio_direction_output(MX25_PIN_CSPI1_SS1, 0);	/* output */
	mxc_set_gpio_direction_output(MX25_PIN_CSPI1_RDY, 0);
	mxc_set_gpio_direction_input(MX25_PIN_UART1_RTS);	/* input */
	mxc_set_gpio_direction_output(MX25_PIN_UART2_RTS, 1);	/* output */
#undef SPEECH_PAD_CTL1
#undef SPEECH_PAD_CTL2
#endif	/* CONFIG_HW_TESTPROG */
}

/*----------------------------------------------------------------------
 * Voice
 */
static inline void iomux_voice_configure(void)
{
#ifdef CONFIG_HW_TESTPROG
	/* mode set */
	mxc_request_iomux(MX25_PIN_KPP_ROW2, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_KPP_ROW3, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_KPP_COL0, MUX_CONFIG_ALT5);
	mxc_request_iomux(MX25_PIN_KPP_COL1, MUX_CONFIG_ALT5);
	/* mxc_request_iomux(MX25_PIN_KPP_COL2, MUX_CONFIG_ALT5); */
	mxc_request_iomux(MX25_PIN_KPP_COL3, MUX_CONFIG_ALT5);

#define VOICE_PAD_CTL	(PAD_CTL_PKE_ENABLE | PAD_CTL_PUE_PUD \
			| PAD_CTL_100K_PU)

	/* pad control */
	mxc_iomux_set_pad(MX25_PIN_KPP_ROW2, VOICE_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_KPP_ROW3, VOICE_PAD_CTL);
	mxc_iomux_set_pad(MX25_PIN_KPP_COL0, VOICE_PAD_CTL | PAD_CTL_ODE_OpenDrain);
	mxc_iomux_set_pad(MX25_PIN_KPP_COL1, VOICE_PAD_CTL | PAD_CTL_ODE_OpenDrain);
	/* mxc_iomux_set_pad(MX25_PIN_KPP_COL2, OICE_PAD_CTL); */
	mxc_iomux_set_pad(MX25_PIN_KPP_COL3, VOICE_PAD_CTL | PAD_CTL_ODE_OpenDrain);

	/* output setting */
	mxc_set_gpio_direction_output(MX25_PIN_KPP_ROW2, 1);
	mxc_set_gpio_direction_output(MX25_PIN_KPP_ROW3, 1);
	mxc_set_gpio_direction_output(MX25_PIN_KPP_COL0, 0);
	mxc_set_gpio_direction_output(MX25_PIN_KPP_COL1, 0);
	/* mxc_set_gpio_direction_output(MX25_PIN_KPP_COL2, 1); */
	mxc_set_gpio_direction_output(MX25_PIN_KPP_COL3, 0);
#undef VOICE_PAD_CTL
#endif	/* CONFIG_HW_TESTPROG */
}

/*----------------------------------------------------------------------
 * UART
 */
static inline void iomux_uart_configure(void)
{
	/* mode set */

	/* UART1 Configuration */
	mxc_request_iomux(MX25_PIN_UART1_RXD, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_UART1_TXD, MUX_CONFIG_FUNC);
	mxc_iomux_set_pad(MX25_PIN_UART1_RXD, PAD_CTL_HYS_SCHMITZ |
			PAD_CTL_PKE_ENABLE | PAD_CTL_PUE_PUD | PAD_CTL_100K_PU);
	mxc_iomux_set_pad(MX25_PIN_UART1_TXD, PAD_CTL_PUE_PUD | PAD_CTL_100K_PD);

	/* UART2 Configuration */
#ifdef CONFIG_HW_TESTPROG
	mxc_request_iomux(MX25_PIN_UART2_RXD, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_UART2_TXD, MUX_CONFIG_FUNC);
	mxc_iomux_set_pad(MX25_PIN_UART2_RXD, PAD_CTL_HYS_SCHMITZ |
			PAD_CTL_PKE_ENABLE | PAD_CTL_PUE_PUD | PAD_CTL_100K_PU);
	mxc_iomux_set_pad(MX25_PIN_UART2_TXD, PAD_CTL_PKE_ENABLE |
				  PAD_CTL_PUE_PUD | PAD_CTL_100K_PD);
#endif	/* CONFIG_HW_TESTPROG */

	/* UART3 Configuration */
#ifdef CONFIG_HW_TESTPROG
	mxc_request_iomux(MX25_PIN_KPP_ROW0, MUX_CONFIG_ALT1);
	mxc_request_iomux(MX25_PIN_KPP_ROW1, MUX_CONFIG_ALT1);
	mxc_request_iomux(MX25_PIN_KPP_COL2, MUX_CONFIG_ALT5);	/* RS485 - GPIO1_GPIO[8]*/
	mxc_request_iomux(MX25_PIN_CSI_MCLK, MUX_CONFIG_ALT5);	/* RS485 - GPIO1_GPIO[8]*/

	mxc_iomux_set_input(MUX_IN_UART3_IPP_UART_RXD_MUX,
				    INPUT_CTL_PATH1);

	mxc_iomux_set_pad(MX25_PIN_KPP_ROW0, PAD_CTL_PKE_ENABLE | PAD_CTL_100K_PU);
	mxc_iomux_set_pad(MX25_PIN_KPP_ROW1, PAD_CTL_PKE_ENABLE | 
				PAD_CTL_PUE_PUD | PAD_CTL_100K_PU);
	mxc_iomux_set_pad(MX25_PIN_KPP_COL2, PAD_CTL_PKE_ENABLE | PAD_CTL_PUE_PUD
			| PAD_CTL_100K_PU | PAD_CTL_ODE_OpenDrain);	/* nRE */
	mxc_iomux_set_pad(MX25_PIN_CSI_MCLK, PAD_CTL_PKE_ENABLE |
				PAD_CTL_PUE_PUD | PAD_CTL_SRE_FAST);

	/* gpio setting */
	mxc_set_gpio_direction_output(MX25_PIN_KPP_COL2, 0);	/* output */
	mxc_set_gpio_direction_output(MX25_PIN_CSI_MCLK, 0);	/* output */
#endif	/* CONFIG_HW_TESTPROG */
}

#ifdef CONFIG_CMD_MMC

u32 *imx_esdhc_base_addr;

int esdhc_gpio_init(void)
{
#ifndef CONFIG_JHRWL3
	u32 interface_esdhc = 0, val = 0;

	interface_esdhc = (readl(CCM_RCSR) & (0x00300000)) >> 20;

	switch (interface_esdhc) {
	case 0:
		imx_esdhc_base_addr = (u32 *)MMC_SDHC1_BASE;
		/* Pins */
		writel(0x10, IOMUXC_BASE + 0x190);	/* SD1_CMD */
		writel(0x10, IOMUXC_BASE + 0x194);	/* SD1_CLK */
		writel(0x00, IOMUXC_BASE + 0x198);	/* SD1_DATA0 */
		writel(0x00, IOMUXC_BASE + 0x19c);	/* SD1_DATA1 */
		writel(0x00, IOMUXC_BASE + 0x1a0);	/* SD1_DATA2 */
		writel(0x00, IOMUXC_BASE + 0x1a4);	/* SD1_DATA3 */
		writel(0x06, IOMUXC_BASE + 0x094);	/* D12 (SD1_DATA4) */
		writel(0x06, IOMUXC_BASE + 0x090);	/* D13 (SD1_DATA5) */
		writel(0x06, IOMUXC_BASE + 0x08c);	/* D14 (SD1_DATA6) */
		writel(0x06, IOMUXC_BASE + 0x088);	/* D15 (SD1_DATA7) */
		writel(0x05, IOMUXC_BASE + 0x010);	/* A14 (SD1_WP) */
		writel(0x05, IOMUXC_BASE + 0x014);	/* A15 (SD1_DET) */

		/* Pads */
		writel(0xD1, IOMUXC_BASE + 0x388);	/* SD1_CMD */
		writel(0xD1, IOMUXC_BASE + 0x38c);	/* SD1_CLK */
		writel(0xD1, IOMUXC_BASE + 0x390);	/* SD1_DATA0 */
		writel(0xD1, IOMUXC_BASE + 0x394);	/* SD1_DATA1 */
		writel(0xD1, IOMUXC_BASE + 0x398);	/* SD1_DATA2 */
		writel(0xD1, IOMUXC_BASE + 0x39c);	/* SD1_DATA3 */
		writel(0xD1, IOMUXC_BASE + 0x28c);	/* D12 (SD1_DATA4) */
		writel(0xD1, IOMUXC_BASE + 0x288);	/* D13 (SD1_DATA5) */
		writel(0xD1, IOMUXC_BASE + 0x284);	/* D14 (SD1_DATA6) */
		writel(0xD1, IOMUXC_BASE + 0x280);	/* D15 (SD1_DATA7) */
		writel(0xD1, IOMUXC_BASE + 0x230);	/* A14 (SD1_WP) */
		writel(0xD1, IOMUXC_BASE + 0x234);	/* A15 (SD1_DET) */

		/*
		 * Set write protect and card detect gpio as inputs
		 * A14 (SD1_WP) and A15 (SD1_DET)
		 */
		val = ~(3 << 0) & readl(GPIO1_BASE + GPIO_GDIR);
		writel(val, GPIO1_BASE + GPIO_GDIR);
		break;
	case 1:
		imx_esdhc_base_addr = (u32 *)MMC_SDHC2_BASE;
		/* Pins */
		writel(0x16, IOMUXC_BASE + 0x0e8);	/* LD8 (SD1_CMD) */
		writel(0x16, IOMUXC_BASE + 0x0ec);	/* LD9 (SD1_CLK) */
		writel(0x06, IOMUXC_BASE + 0x0f0);	/* LD10 (SD1_DATA0) */
		writel(0x06, IOMUXC_BASE + 0x0f4);	/* LD11 (SD1_DATA1) */
		writel(0x06, IOMUXC_BASE + 0x0f8);	/* LD12 (SD1_DATA2) */
		writel(0x06, IOMUXC_BASE + 0x0fc);	/* LD13 (SD1_DATA3) */
		writel(0x02, IOMUXC_BASE + 0x120);	/* CSI_D2 (SD1_DATA4) */
		writel(0x02, IOMUXC_BASE + 0x124);	/* CSI_D3 (SD1_DATA5) */
		writel(0x02, IOMUXC_BASE + 0x128);	/* CSI_D4 (SD1_DATA6) */
		writel(0x02, IOMUXC_BASE + 0x12c);	/* CSI_D5 (SD1_DATA7) */

		/* Pads */
		writel(0xD1, IOMUXC_BASE + 0x2e0);	/* LD8 (SD1_CMD) */
		writel(0xD1, IOMUXC_BASE + 0x2e4);	/* LD9 (SD1_CLK) */
		writel(0xD1, IOMUXC_BASE + 0x2e8);	/* LD10 (SD1_DATA0) */
		writel(0xD1, IOMUXC_BASE + 0x2ec);	/* LD11 (SD1_DATA1) */
		writel(0xD1, IOMUXC_BASE + 0x2f0);	/* LD12 (SD1_DATA2) */
		writel(0xD1, IOMUXC_BASE + 0x2f4);	/* LD13 (SD1_DATA3) */
		writel(0xD1, IOMUXC_BASE + 0x318);	/* CSI_D2 (SD1_DATA4) */
		writel(0xD1, IOMUXC_BASE + 0x31c);	/* CSI_D3 (SD1_DATA5) */
		writel(0xD1, IOMUXC_BASE + 0x320);	/* CSI_D4 (SD1_DATA6) */
		writel(0xD1, IOMUXC_BASE + 0x324);	/* CSI_D5 (SD1_DATA7) */
		break;
	default:
		break;
	}
#endif	/* CONFIG_JHRWL3 */
	return 0;
}

/*----------------------------------------------------------------------
 * NAND-Flash
 */
void iomux_nand_configure(void)
{
	mxc_request_iomux(MX25_PIN_NF_CE0, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_NFWE_B, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_NFRE_B, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_NFALE, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_NFCLE, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_NFWP_B, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_NFRB, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_D0, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_D1, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_D2, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_D3, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_D4, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_D5, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_D6, MUX_CONFIG_FUNC);
	mxc_request_iomux(MX25_PIN_D7, MUX_CONFIG_FUNC);

	/* NF_CE0 */
	mxc_iomux_set_input(MUX_IN_CSPI1_IPP_IND_SS3_B,
				    INPUT_CTL_PATH0);

	mxc_iomux_set_pad(MX25_PIN_NF_CE0, PAD_CTL_PUE_PUD);
	mxc_iomux_set_pad(MX25_PIN_NFWE_B, PAD_CTL_PUE_PUD);
	mxc_iomux_set_pad(MX25_PIN_NFRE_B, PAD_CTL_PUE_PUD | 
				PAD_CTL_PKE_ENABLE | PAD_CTL_100K_PU);
	mxc_iomux_set_pad(MX25_PIN_NFALE, PAD_CTL_PUE_PUD);
	mxc_iomux_set_pad(MX25_PIN_NFCLE, PAD_CTL_PUE_PUD);
	mxc_iomux_set_pad(MX25_PIN_NFWP_B, PAD_CTL_PUE_PUD);
	mxc_iomux_set_pad(MX25_PIN_NFRB, PAD_CTL_PUE_PUD | 
				PAD_CTL_PKE_ENABLE | PAD_CTL_100K_PU);
	mxc_iomux_set_pad(MX25_PIN_D0, PAD_CTL_PKE_ENABLE);
	mxc_iomux_set_pad(MX25_PIN_D1, PAD_CTL_PKE_ENABLE);
	mxc_iomux_set_pad(MX25_PIN_D2, PAD_CTL_PKE_ENABLE);
	mxc_iomux_set_pad(MX25_PIN_D3, PAD_CTL_PKE_ENABLE);
	mxc_iomux_set_pad(MX25_PIN_D4, PAD_CTL_PKE_ENABLE);
	mxc_iomux_set_pad(MX25_PIN_D5, PAD_CTL_PKE_ENABLE);
	mxc_iomux_set_pad(MX25_PIN_D6, PAD_CTL_PKE_ENABLE);
	mxc_iomux_set_pad(MX25_PIN_D7, PAD_CTL_PKE_ENABLE);
}

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Board Initialize function
 */
int dram_init(void)
{
	gd->bd->bi_dram[0].start = PHYS_SDRAM_1;
	gd->bd->bi_dram[0].size  = PHYS_SDRAM_1_SIZE;
#ifdef CONFIG_JHRWL3_0TP_BOARD
	gd->bd->bi_dram[1].start = PHYS_SDRAM_2;
	gd->bd->bi_dram[1].size  = PHYS_SDRAM_2_SIZE;
#endif	/* CONFIG_JHRWL3_0TP_BOARD */

	return 0;
}

int board_mmc_init(void)
{
#ifndef CONFIG_JHRWL3
	if (!esdhc_gpio_init())
		return fsl_esdhc_mmc_init(gd->bd);
	else
		return -1;
#else
	return 0;
#endif	/* CONFIG_JHRWL3 */
}
#endif

void spi_io_init(struct imx_spi_dev_t *dev)
{
	switch (dev->base) {
	case CSPI1_BASE:
		iomux_speech_configure();
		break;
	default:
		break;
	}
}

/* Cpu Clock Setting */
static int adc_sampling_get(int ch);
static void board_cpufreq_setting(void)
{
#if 0
#define POWER_ON_VOLTAGE	(1000)	/* mV */
	int inaux[2];
	int i;
#endif
	u32 mpctl, cctl;
	u32 cgcr0, cgcr1, cgcr2;

#if 0
	for (i = 0; i < 2; i++) {
		inaux[i] = adc_sampling_get(i);	/* 0:INAUX0, 1:INAUX1 */
	}

	if (inaux[1] >= POWER_ON_VOLTAGE) {
		/* 
		 * CPU Clock ARM=399 AHB=133
		 *  - MPCTL=0x800BAC01 CCTL=0x20034000
		 */
		 mpctl = 0x800BAC01;
		 cctl  = 0x20034000;
	} else if (inaux[0] >= POWER_ON_VOLTAGE) {
		/* 
		 * CPU Clock ARM=266 AHB=133
		 *  - MPCTL=0x800B2C01 CCTL=0x50030000
		 */
		 mpctl = 0x800B2C01;
		 cctl  = 0x50030000;
	} else {
		/* Error Voltage(default clock) */
		 goto end;
	}
#else
	/* 
	 * CPU Clock ARM=399 AHB=133
	 *  - MPCTL=0x800BAC01 CCTL=0x20034000
	 */
	 mpctl = 0x800BAC01;
	 cctl  = 0x20034000;
#endif

	cgcr0 = (u32)__REG(CCM_CGR0);
	__REG(CCM_CGR0) = (u32)(cgcr0 & ~(0x0000FFFF));
	cgcr1 = (u32)__REG(CCM_CGR1);
	__REG(CCM_CGR1) = (u32)(cgcr1 & ~(0xFFFFFFFF));
	cgcr2 = (u32)__REG(CCM_CGR2);
	__REG(CCM_CGR2) = (u32)(cgcr2 & ~(0x000FFFFF));

	__REG(CCM_MPCTL) = (u32)mpctl;
	__REG(CCM_CCTL)  = (u32)cctl | (1 << 27);

	__REG(CCM_CGR0) = (u32)cgcr0;
	__REG(CCM_CGR1) = (u32)cgcr1;
	__REG(CCM_CGR2) = (u32)cgcr2;
end:
	return;
}

int board_init(void)
{
	board_cpufreq_setting();

	setup_soc_rev();

	iomux_nand_configure();
#if 0
	iomux_clko_configure();		/* IC Clock Enable */
#endif

	iomux_uart_configure();

	iomux_ethphy_configure();
	iomux_i2c_configure();

	iomux_switch_configure();
	iomux_led_configure();
	iomux_voice_configure();

	iomux_lcd_configure();
	iomux_lcdbl_configure();

	iomux_wlan_configure();

	gd->bd->bi_arch_number = MACH_TYPE_MX25_3DS;    /* board id for linux */
	gd->bd->bi_boot_params = 0x80000100;    /* address of boot parameters */

	return 0;
}

#ifdef BOARD_LATE_INIT
#if 1	/* Add itaya 2012/02/21 */
extern void fw_eeprom_init (void);
extern char *fw_eeprom_macaddr_read (void);
#endif	/* Add itaya 2012/02/21 */

int board_late_init(void)
{
	u8 reg[4];
#ifndef CONFIG_WRITE_BOOTLOADER
	char *ethaddr;
#endif	/* CONFIG_WRITE_BOOTLOADER */

	/* Turn PMIC On*/
	reg[0] = 0x09;

	if (i2c_write(0x54, 0x02, 1, reg, 1) < 0)
		printf("PMIC error...\n");

#ifdef CONFIG_WRITE_BOOTLOADER
	if (getenv("ethaddr") == NULL) {
		setenv("ethaddr", MK_STR(CONFIG_ETHADDR));
	}
	setenv("bootcmd", CONFIG_BOOTCOMMAND);
#else	/* CONFIG_WRITE_BOOTLOADER */
	fw_eeprom_init();
	ethaddr = fw_eeprom_macaddr_read();
	if (ethaddr) {
		eth_setenv_enetaddr("ethaddr", ethaddr);
	} else {
		if (getenv("ethaddr") == NULL) {
			setenv("ethaddr", MK_STR(CONFIG_ETHADDR));
		}
	}
#endif	/* CONFIG_WRITE_BOOTLOADER */

	return 0;
}
#endif

#include <miiphy.h>

int board_eth_init(bd_t *bis)
{
	int rc = -ENODEV;

	cpu_eth_init(bis);

#ifdef CONFIG_JHRWL3_0TP_BOARD
	/* Auto Negotiation Setup */
	{
#define PHY_SMSC_CTLSTS		17		/* Mode/Status Register */
# define CTLSTS_EDPWRDOWN	(1 << 13)	/* EDPWRDOWN */
#define PHY_SMSC_ISF		29		/* Interrupt Source Flags */
#define PHY_SMSC_ADDR		0x0

		char *devname = miiphy_get_current_dev();
		u16  value;

		/* Hardware reset */
		mxc_set_gpio_dataout(MX25_PIN_D12, 0);
		udelay(25 * 1000);		/* spec says 25msec min */
		mxc_set_gpio_dataout(MX25_PIN_D12, 1);

		/* Fec Driver Setup */
		fec_setup(devname);

		/* Software Reset */
		miiphy_reset(devname, PHY_SMSC_ADDR);

		/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		 * Linux Initialize Function
		 */
		miiphy_read(devname, PHY_SMSC_ADDR, PHY_SMSC_CTLSTS, &value);

		/* Enable energy detect mode for this SMSC Transceivers */
		miiphy_write(devname, PHY_SMSC_ADDR, PHY_SMSC_CTLSTS,
				(value | CTLSTS_EDPWRDOWN));

		/* Create Interrupt Source */
		miiphy_read(devname, PHY_SMSC_ADDR, PHY_SMSC_ISF, &value);
	}
#endif	/* CONFIG_JHRWL3_0TP_BOARD */
	rc = 0;

	return rc;
}

/* bootmode_args setting */
int board_testmode_set(void)
{
#define ECOCUTE_CH	(0)
#define POWERCON_CH	(1)
#define MODE_CHECK_USEC	(100)
#define MODE_CHECK_TIME	((5 * 1000 * 1000) / MODE_CHECK_USEC)
	char bootmode_args[] = "bootmode_args";
	int  bootmode = 0;	/* 0: normal, 1:testmode, 2:sjis, 3:jet */
	int  i;

	/*
	 * test mode check
	 *  - mode is 5sec over 'home' is low & 'bus botton' is high -> jet
	 *  - mode is 5sec over 'home + bus botton' + adc 0 is high -> sjis
	 *  - mode is 5sec over 'drive botton' -> test mode
	 *
	 * mode priority is 'jet' > sjis' > 'test' > 'normal'
	 */
#if 1	/* L5 support */
	if ((mxc_gpio_get(MX25_PIN_D9) == 1) && (mxc_gpio_get(MX25_PIN_D10) == 0)) {
		bootmode = 3;	/* jet */
	} else
#endif	/* L5 support */
	if (((mxc_gpio_get(MX25_PIN_D9) == 0) && (mxc_gpio_get(MX25_PIN_D10) == 0)) &&
		(adc_sampling_get(ECOCUTE_CH) >= 1000)) {
		bootmode = 2;	/* sjis */
	} else if (mxc_gpio_get(MX25_PIN_D11) == 0) {
		bootmode = 1;	/* test */
	} else {
		bootmode = 0;	/* normal */
		goto bootset;
	}

	/* check 5sec */
	for (i = 0; i < MODE_CHECK_TIME; i++) {
#if 1	/* L5 support */
		/* mode is jet mode check */
		if ((bootmode == 3) && ((mxc_gpio_get(MX25_PIN_D9) != 1) || 
					(mxc_gpio_get(MX25_PIN_D10) != 0))) {
			bootmode = 0;
			break;
		}
#endif	/* L5 support */
		/* mode is sjis mode check */
		if ((bootmode == 2) && ((mxc_gpio_get(MX25_PIN_D9) != 0) || 
					(mxc_gpio_get(MX25_PIN_D10) != 0))) {
			bootmode = 0;
			break;
		}
		/* mode is test mode chceck */
		if ((bootmode == 1) && (mxc_gpio_get(MX25_PIN_D11) != 0)) {
			bootmode = 0;
			break;
		}
		udelay(MODE_CHECK_USEC);
	}

bootset:
	/* enviroment bootmode argument setting */
	switch (bootmode) {
	case 1:
		setenv(bootmode_args, "test");

		/* detect test mode */
		LED_LIGHT(MX25_PIN_CSI_D2, 1);
		LED_LIGHT(MX25_PIN_CSI_D3, 1);
		LED_LIGHT(MX25_PIN_CSI_D4, 1);
		LED_LIGHT(MX25_PIN_CSI_D5, 1);
		LED_LIGHT(MX25_PIN_CSI_D6, 1);
		LED_LIGHT(MX25_PIN_CSI_D7, 1);
		LED_LIGHT(MX25_PIN_CSI_D8, 1);
		LED_LIGHT(MX25_PIN_CSI_VSYNC, 1);
		LED_LIGHT(MX25_PIN_CSI_HSYNC, 1);
		break;
	case 2:
		/* last check */
		if (adc_sampling_get(ECOCUTE_CH) >= 1000)
			setenv(bootmode_args, "sjis");
		else
			setenv(bootmode_args, "normal");

		break;
#if 1	/* L5 support */
	case 3:
		setenv(bootmode_args, "jet");

		break;
#endif	/* L5 support */
	case 0:
	default:
		setenv(bootmode_args, "normal");
		break;
	}

	return 0;
}

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Board Check function
 */
int checkboard(void)
{
	printf("Board: i.MX25 - Solar-System JHRWL3 board\n");

	return 0;
}

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * eeprom write protect function
 */
#if defined(CONFIG_SYS_EEPROM_WREN) & defined(CONFIG_SYS_I2C_EEPROM_ADDR)
extern int eeprom_write_enable (unsigned dev_addr, int state);

int eeprom_write_enable (unsigned dev_addr, int state)
{
	if (dev_addr != CONFIG_SYS_I2C_EEPROM_ADDR) {
		printf("\nNot Supported EEPROM - addr is %#x\n", dev_addr);
		return -1;
	}

	if (state) {	/* write protect */
		mxc_set_gpio_dataout(MX25_PIN_UART1_CTS, 0);
	} else {
		mxc_set_gpio_dataout(MX25_PIN_UART1_CTS, 1);
	}

	return 0;
}
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * watchdog function
 */

#ifndef __REG16
# define __REG16(base)	*(volatile u16 *)(base)
#endif

#ifndef WDOG_BASE
#define WDOG_BASE	0x53FDC000
#endif

#define WDOG_WCR	(WDOG_BASE + 0x00)
# define WCR_WOE_BIT		(1 << 6)
# define WCR_WDA_BIT		(1 << 5)
# define WCR_SRS_BIT		(1 << 4)
# define WCR_WRE_BIT		(1 << 3)
# define WCR_WDE_BIT		(1 << 2)
# define WCR_WDBG_BIT		(1 << 1)
# define WCR_WDZST_BIT		(1 << 0)

#define WDOG_WSR	(WDOG_BASE + 0x02)
#define WDOG_WRSR	(WDOG_BASE + 0x04)
#define WDOG_WICR	(WDOG_BASE + 0x06)
#define WDOG_WMCR	(WDOG_BASE + 0x08)

#define WDOG_SEC_TO_COUNT(s)	((s) << 8)
#define WDOG_COUNT_TO_SEC(c)	((c >> 8))

void wdog_clk_enable(void)
{
#define MXC_CCM_CGCR2			(CCM_BASE + 0x14)
#define MXC_CCM_CGCR2_WDOG_OFFSET	(51-32)

	u32 reg = __REG(MXC_CCM_CGCR2);

	reg |= 1 << MXC_CCM_CGCR2_WDOG_OFFSET;

	*(volatile u32 *)MXC_CCM_CGCR2 = (u32)reg;
}

void wdog_ping(void)
{
#define WDT_MAGIC_1	(0x5555)
#define WDT_MAGIC_2	(0xAAAA)
	/* issue the service sequence instructions */
	*(volatile u16 *)(WDOG_WSR) = (u16)WDT_MAGIC_1;
	*(volatile u16 *)(WDOG_WSR) = (u16)WDT_MAGIC_2;
}

void wdog_config(void)
{
	u16 val;

	val = (u16)__REG16(WDOG_WCR);
	val |= 0xFF00 | WCR_WOE_BIT | WCR_WDA_BIT | WCR_SRS_BIT;
	/* enable suspend WDT */
	val |= WCR_WDBG_BIT;
	/* generate reset if wdog times out */
	val &= ~WCR_WRE_BIT;
	*(volatile u16 *)(WDOG_WCR) = (u16)(val);
}

void wdog_set_timeout(u16 timeout)
{
	u16 val;

	val = (u16)__REG16(WDOG_WCR);
	val = (val & 0x00FF) | WDOG_SEC_TO_COUNT(timeout);
	*(volatile u16 *)(WDOG_WCR) = (u16)(val);
}

void wdog_enable(void)
{
	u16 val;

	val = (u16)__REG16(WDOG_WCR);
	val |= WCR_WDE_BIT;
	*(volatile u16 *)(WDOG_WCR) = (u16)val;
}

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * ADC function (INAUX0)
 *  Linux Driver is "driver/mxc/adc/imx_adc.c"
 */
#define ADC_BASE	TSC_BASE
#define ADC_TGCR	(TSC_BASE + 0x000)
# define TGCR_IPG_CLK_EN       (1 << 0)
# define TGCR_TSC_RST          (1 << 1)
# define TGCR_POWER_SAVE       (0x1 << 8)
#define ADC_GCQFIFO	(ADC_BASE + 0x800)
#define ADC_GCQCR	(ADC_BASE + 0x804)
# define CQCR_FQS	(1 << 2)
#define ADC_GCQSR	(ADC_BASE + 0x808)
# define CQSR_EOQ	(1 << 1)
# define CQSR_EMPT	(1 << 13)
#define ADC_GCQMR	(ADC_BASE + 0x80C)
#define ADC_GCC0	(ADC_BASE + 0x840)
# define GENERAL_ADC_GCC(n)	(0x174c + ((n+1) << 4))

#define	REF_VOLTAGE		(3300)		/* 3.3 V */
#define ADC_SAMPLE		(4)

static int adc_sampling_get(int ch)
{
	int lastitemid;
	int voltage = 0;
	u32 val;
	u32 result[16];
	u32 data_num = 0;

	if (ch >= 3)	/* support channel is 0`2 */
		return -1;

	/* disable adc ipgclk in CCM */
	val = __REG(CCM_CGR2);
	val &= ~(1 << 13);
	__REG(CCM_CGR2) = val;

	/* disable adc ipgclken bit */
	val = __REG(ADC_TGCR);
	val &= ~TGCR_IPG_CLK_EN;
	__REG(ADC_TGCR) = val;

	/* set ipgclk ctl by adc enable bit */
	val = __REG(CCM_CCTL);
	val |=  (1 << 15);	
	__REG(CCM_CCTL) = val;

	/* enable adc ipgclk in CCM */
	val = __REG(CCM_CGR2);
	val |=  (1 << 13);
	__REG(CCM_CGR2) = val;

	/* enable adc ipgclken bit */
	val = __REG(ADC_TGCR);
	val |= TGCR_IPG_CLK_EN;	
	__REG(ADC_TGCR) = val;

	/* self-reset adc */
	val = __REG(ADC_TGCR);
	val |= TGCR_TSC_RST;	
	__REG(ADC_TGCR) = val;

	/* wait until self-reset is done */
	while(__REG(ADC_TGCR) & TGCR_TSC_RST)
		continue;

	/* adc in power saving mode */
	val = __REG(ADC_TGCR);
	val |= TGCR_POWER_SAVE;	
	__REG(ADC_TGCR) = val;

	/* initialize */
	lastitemid = 0;
	val = (0xf << 8) | (lastitemid << 4) | 0x2;
	__REG(ADC_GCQCR) = val;

	val = GENERAL_ADC_GCC(ch);
	val |= (3 << 16) | (16 << 24);
	__REG(ADC_GCC0) = val;

	/* sampling start */
	val = __REG(ADC_GCQCR);
	val |= CQCR_FQS;
	__REG(ADC_GCQCR) = val;

	/* sampling wait */
	while (!(__REG(ADC_GCQSR) & CQSR_EOQ))
		continue;

	/* sampling stop */
	val = __REG(ADC_GCQCR);
	val &= ~CQCR_FQS;
	__REG(ADC_GCQCR) = val;
	val = __REG(ADC_GCQSR);
	val |= CQSR_EOQ;
	__REG(ADC_GCQSR) = val;

	/* sampling result get */
	while (!(__REG(ADC_GCQSR) & CQSR_EMPT)) {

		result[data_num] = __REG(ADC_GCQFIFO) >> 4;

		if (data_num < ADC_SAMPLE)
			voltage += (REF_VOLTAGE * result[data_num]) / 4096;

		data_num++;
		if (data_num >= 16)
			break;
	}
	voltage /= ADC_SAMPLE;

	return voltage;
}

#if defined(CONFIG_SYS_DRAM_TEST)
typedef struct {
	unsigned int start;
	unsigned int end;
} testdram_t;

#define END_POINT(start)	(start + (1 * 1024 * 1024))

#define DRAM_TEST_1		(PHYS_SDRAM_2)
#define DRAM_TEST_2		(PHYS_SDRAM_2 + (64 * 1024 * 1024))
#define DRAM_TEST_END		(0)

static testdram_t dram_test[] = {
	/* start		end	*/
	{ DRAM_TEST_1,		END_POINT(DRAM_TEST_1) },
	{ DRAM_TEST_2	,	END_POINT(DRAM_TEST_2) },
	{ DRAM_TEST_END,	DRAM_TEST_END}
};

int testdram (void)
{
	int i;
	unsigned int *pstart;
	unsigned int *pend;
	unsigned int *p;

	for (i = 0; i < 2; i++) {
		pstart = dram_test[i].start;
		pend   = dram_test[i].end;

		if (*pstart == DRAM_TEST_END)
			break;

		printf("SDRAM Test Aread %p - %p\n",
				pstart, pend);

		printf("SDRAM test phase 1:\n");
		for (p = pstart; p < pend; p++)
			*p = 0xaaaaaaaa;
	
		for (p = pstart; p < pend; p++) {
			if (*p != 0xaaaaaaaa) {
				printf ("SDRAM test fails at: %08x\n", (unsigned int) p);
				return 1;
			}
		}
	
		printf("SDRAM test phase 2:\n");
		for (p = pstart; p < pend; p++)
			*p = 0x55555555;
	
		for (p = pstart; p < pend; p++) {
			if (*p != 0x55555555) {
				printf ("SDRAM test fails at: %08x\n", (unsigned int) p);
				return 1;
			}
		}
	}

	printf("SDRAM test passed.\n");

	return 0;
}
#endif	/* CONFIG_SYS_DRAM_TEST */

