/*
 * (C) Copyright 2011
 * Working Solar System Production
 *  
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 *
 *
 *
 */

#include <common.h>
#include <command.h>
#include <asm/io.h>
#include <asm/errno.h>
#include <asm/arch/mx25.h>
#include <asm/arch/mx25-regs.h>
#include <asm/arch/mx25_pins.h>
#include <asm/arch/iomux.h>
#include <asm/arch/gpio.h>

#if !defined(CONFIG_CMD_NAND) | !defined(CONFIG_CMD_EEPROM)
#error "Not Supported .. Command EEPROM/NAND"
#endif	/* CONFIG_CMD_NAND */

/* #define CMD_FLBOOT_DEBUG */
#undef CMD_FLBOOT_DEBUG
#ifdef	CMD_FLBOOT_DEBUG
# define PRINTF(fmt,args...)	printf (fmt ,##args)
#else
# define PRINTF(fmt,args...)
#endif

#ifndef CONFIG_SYS_LOAD_ADDR
# define CONFIG_SYS_LOAD_ADDR	0x80800000
#endif	/* CONFIG_SYS_LOAD_ADDR */

#ifndef XMK_STR
# define XMK_STR(x)	#x
#endif	/* XMK_STR */

#ifndef MK_STR
# define MK_STR(x)	XMK_STR(x)
#endif	/* MK_STR */

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * define
 */
/* Saved EEPROM Boot Start Infomation */
#define START_STS_MASK	(0x00ff)
# define START_FLAG_OK		(0x0000)	/* Start ok */
# define START_FLAG_UPDATE	(0x0001)	/* update - reboot */
# define START_FLAG_FIRST	(0x0002)	/* update */
# define START_FLAG_BACKUP	(0x0003)

#define START_SIDE_MASK	(0xff00)
# define START_SIDE_A		(0x0a00)
# define START_SIDE_B		(0x0b00)

/* EEPROM */
#define MAGIC_JHRWL3		(0x09510103)
#define MAGIC_JHRWL4		(0x09510104)
#if 1	/* L5 support */
#define MAGIC_JHRWL5		(0x09510105)
#endif	/* L5 support */
#if 1	/* L6 support */
#define MAGIC_JHRWL6		(0x09510106)
#endif	/* L6 support */
#if 1   /* L7 support */
#define MAGIC_JHRWL7            (0x09510107)
#endif  /* L7 support */

#define KERNEL_A_SIDE		0x00200000
#define KERNEL_B_SIDE		0x00A00000
#define KERNEL_C_SIDE		0x01200000
#define KERNEL_D_SIDE		0x01A00000

#define FW_DEFAULT_VERSION	"0000"
#define FW_DEFAULT_BUILD	"0000"

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * global
 */
#if 0	/* Delete itaya 2012/02/21 */
static int default_param = 1;
#endif	/* Delete itaya 2012/02/21 */

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * struncture
 */

/* EEPROM MAP Infomation */
struct boot_fw {
	u32	kernel_addr;
	u8	version[4];
	u8	build_ver[4];
};

/* EEPROM MAP Infomation */
#pragma pack(push, 1)
struct boot_fw_eeprom {
	u32		magic;		/* Magic Code */
	u8		ethaddr[6];	/* Mac Address */
	u8		wlanaddr[6];	/* WLAN Mac Address */
	uint16_t	bootflags;	/* Bootflags */
	struct boot_fw	side_a;
	struct boot_fw	side_b;
	u8		serial_number[8];
	u8		mon_id[11];
	u8		reserved[63];
	u32		crc;		/* CRC32 */
};
#pragma pack(pop)	/* pragma */

enum {
	EEPROM_BANK_0 = 0,
	EEPROM_BANK_1,
	EEPROM_BANK_MAX
};

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * extern
 */
extern int eeprom_read (unsigned, unsigned, uchar *, unsigned);
extern int eeprom_write (unsigned, unsigned, uchar *, unsigned);

extern int run_command (const char *cmd, int flag);

extern void mxc_fec_set_mac_from_env(char *mac_addr);

extern void wdog_clk_enable(void);
extern void wdog_ping(void);
extern void wdog_config(void);
extern void wdog_set_timeout(u16 timeout);
extern void wdog_enable(void);

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * macro
 */
inline static
off_t __eeprom_off_conv(char *top, char *ptr, int bank)
{
	return (off_t)((ptr - top) + (bank * sizeof(struct boot_fw_eeprom)));
}
#define eeprom_off_conv(top, ptr, bank)	\
	__eeprom_off_conv((char *)top, (char *)ptr, bank)

/* Rapper EEPROM */
#ifndef CONFIG_SYS_EEPROM_PAGE_WRITE_DELAY_MS
inline static
int __eeprom_read(unsigned dev_addr, unsigned offset, uchar *buffer, unsigned cnt)
{
	int i, ret = 0;

	for (i = 0; i < cnt; i++) {
		if (ret = eeprom_read(dev_addr, (offset + i), (buffer + i), 1))
			break;
		udelay(1000);
	}

	return ret;
}

inline static
int __eeprom_write(unsigned dev_addr, unsigned offset, uchar *buffer, unsigned cnt)
{
	int i, ret = 0;

	for (i = 0; i < cnt; i++) {
		if (ret = eeprom_write(dev_addr, (offset + i), (buffer + i), 1))
			break;
		udelay(10*1000);
	}

	return ret;
}
#else
#define __eeprom_read(dev_addr, offset, buffer, cnt)	eeprom_read(dev_addr, offset, buffer, cnt)
#define __eeprom_write(dev_addr, offset, buffer, cnt)	eeprom_write(dev_addr, offset, buffer, cnt)
#endif	/* CONFIG_SYS_EEPROM_PAGE_WRITE_DELAY_MS */

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * local functions
 */

/*
 * Mac Address setting
 */
static void fw_setenv_macaddr(u8 *ethaddr, char *envname)
{
	char tmp_addr[18];

	sprintf(tmp_addr, "%02X:%02X:%02X:%02X:%02X:%02X",
		*ethaddr, *(ethaddr + 1), *(ethaddr + 2),
		*(ethaddr + 3), *(ethaddr + 4), *(ethaddr + 5));

	if (memcmp(envname, "ethaddr", 7) && memcmp(envname, "wlanaddr", 8))
		printf(" Enviroment Name error : %s\n", envname);
	else {
		setenv(envname, tmp_addr);
		mxc_fec_set_mac_from_env(tmp_addr);
	}
}

/*
 * rootfs bootargs setting
 */
static void fw_setenv_rootfs(int load)
{
#if 0
# define MOUNTED_OPTION	"rw"
#else
# define MOUNTED_OPTION	"ro"
#endif

	char rootfs_args[64];
	char rootfs_part[32];

	if (!load) {	/* normal rootfs */
		sprintf(rootfs_part, "root=ubi0:rootfs_a");
	} else {	/* backup rootfs */
		sprintf(rootfs_part, "root=ubi0:rootfs_b");
	}
	sprintf(rootfs_args, "ubi.mtd=8,1024 %s rootfstype=ubifs %s",
				rootfs_part, MOUNTED_OPTION);

	setenv("rootfs_args", rootfs_args);
}

/*
 * loadaddr setting
 */
static void fw_setenv_loadaddr(u32 loadaddr)
{
	char env_loadaddr[16];

	/* loadaddr setting */
	sprintf(env_loadaddr, "0x%08x", loadaddr);
	setenv("loadaddr", env_loadaddr);
}

/*
 * bootargs setting
 */
static void fw_setenv_bootargs(void)
{
	char flbootargs[128];
	char * rootfs_args;
	char * bootmode_args;
	char * debug_args;

#if 0	/* IP Addr Setting */
	char ipaddr[32];
	sprintf(ipaddr, "ip=%s", getenv("ipaddr"));
#else
	char ipaddr[]="";
#endif

	rootfs_args = getenv("rootfs_args");
	bootmode_args = getenv("bootmode_args");
	debug_args = getenv("debug_args");

	if ((rootfs_args != NULL) && (bootmode_args != NULL)) {
		if (debug_args) {
			sprintf(flbootargs, "console=ttymxc0,115200 %s bootcmd=%s %s %s",
				rootfs_args, bootmode_args, ipaddr, debug_args);
		} else {
			sprintf(flbootargs, "console=ttymxc0,115200 %s bootcmd=%s %s",
				rootfs_args, bootmode_args, ipaddr);
		}

		if (memcmp(bootmode_args, "test", 4) == 0) {
			printf("+++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
			printf(" Test mode start ...                                   \n");
			printf("  - bootargs : %s\n", flbootargs);
			printf("+++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
		}

		setenv("bootargs", flbootargs);
	}
}

#if 1	/* Add itaya 2012/02/21 */
static struct boot_fw_eeprom eeprom_fw_info;

void fw_eeprom_init (void)
{
	int bank;
	struct boot_fw_eeprom eeprom_info[EEPROM_BANK_MAX];
	struct boot_fw_eeprom *fw_info = NULL;
	u32 eeplen = sizeof(struct boot_fw_eeprom);

	memset(&eeprom_fw_info, 0, eeplen);

	for (bank = 0; bank < EEPROM_BANK_MAX; bank++) {
		fw_info = &eeprom_info[bank];

		memset(fw_info, 0, eeplen);

		/* eeprom read */
		if (__eeprom_read(CONFIG_SYS_DEF_EEPROM_ADDR, (bank * eeplen),
				(uchar *)fw_info, eeplen) < 0) {
			printf(" EEPROM Bank %d: Read Error(%#x)\n", bank, (bank * eeplen));
			memset(fw_info, 0, eeplen);
		}
	}

	for (bank = 0; bank < EEPROM_BANK_MAX; bank++) {
		fw_info = &eeprom_info[bank];

		/* magic code check */
		printf("Bank %d - EEPROM Magic Check ....", bank);
#if 1	/* L5 support */
		if ((fw_info->magic & 0xffffff00) != (MAGIC_JHRWL3 & 0xffffff00)) {
			printf("Bad Magic Code(%#x)\n",  fw_info->magic);
			continue;
		}
#else	/* L5 support */
		if ((fw_info->magic != MAGIC_JHRWL3) && (fw_info->magic != MAGIC_JHRWL4)) {
			printf("Bad Magic Code(%#x)\n",  fw_info->magic);
			continue;
		}
#endif	/* L5 support */
		printf("OK(%#x)\n", fw_info->magic);

		/* crc32 check */
		printf("Bank %d - EEPROM CRC Verify ....", bank);
		if (crc32(0, (uchar *)fw_info, (eeplen-sizeof(fw_info->crc))) != fw_info->crc) {
			printf("Bad CRC(%#x)\n",  fw_info->magic);
			continue;
		}
		printf("OK\n");
		break;
	}

	if (bank == EEPROM_BANK_0) {
		fw_info = &eeprom_info[EEPROM_BANK_0];

		if (memcmp (fw_info, &eeprom_info[EEPROM_BANK_1], eeplen) != 0) {
			/* Write BANK 1 */
			if (__eeprom_write(CONFIG_SYS_DEF_EEPROM_ADDR, eeplen,
					(uchar *)fw_info, eeplen) < 0) {
				printf(" EEPROM Bank 1 : write error...\n");
			}
		}
	} else if (bank == EEPROM_BANK_1) {
		fw_info = &eeprom_info[EEPROM_BANK_1];

			/* Write BANK 0 */
		if (__eeprom_write(CONFIG_SYS_DEF_EEPROM_ADDR, 0,
				(uchar *)fw_info, eeplen) < 0) {
			printf(" EEPROM Bank 0 : write error...\n");
		}
	} else {
		printf("No Data\n");

		printf("Set EEPROM Default Data\n");
		for (bank = 0; bank < EEPROM_BANK_MAX; bank++) {
			fw_info = &eeprom_info[bank];

			memset(fw_info, 0, eeplen);

			fw_info->magic = MAGIC_JHRWL4;
			eth_parse_enetaddr(MK_STR(CONFIG_ETHADDR), fw_info->ethaddr);
			fw_info->bootflags = (START_SIDE_A | START_FLAG_UPDATE);
			fw_info->side_a.kernel_addr = KERNEL_A_SIDE;
			memcpy(fw_info->side_a.version, FW_DEFAULT_VERSION, 4);
			memcpy(fw_info->side_a.build_ver, FW_DEFAULT_BUILD, 4);
			fw_info->side_b.kernel_addr = KERNEL_B_SIDE;
			memcpy(fw_info->side_b.version, FW_DEFAULT_VERSION, 4);
			memcpy(fw_info->side_b.build_ver, FW_DEFAULT_BUILD, 4);
			fw_info->crc = crc32(0, (uchar *)fw_info,
					(sizeof(*fw_info) - sizeof(fw_info->crc)));

			if (__eeprom_write(CONFIG_SYS_DEF_EEPROM_ADDR, (bank * eeplen),
				(uchar *)fw_info, eeplen) < 0) {
				printf(" EEPROM Bank %d: Write Error(%#x)\n", bank, (bank * eeplen));
			}
		}
		fw_info = &eeprom_info[EEPROM_BANK_0];
	}

	memcpy(&eeprom_fw_info, fw_info, eeplen);
}

char *fw_eeprom_macaddr_read (void)
{
	u8 zero_mac[6];
	u8 ff_mac[6];
	u8 def_mac[6];

	memset(zero_mac, 0, 6);
	memset(ff_mac, 0xff, 6);
	eth_parse_enetaddr(MK_STR(CONFIG_ETHADDR), def_mac);

	if ((memcmp (eeprom_fw_info.ethaddr, zero_mac, 6) == 0) ||
		(memcmp (eeprom_fw_info.ethaddr, ff_mac, 6) == 0)   ||
		(memcmp (eeprom_fw_info.ethaddr, def_mac, 6) == 0)) {
		return NULL;
	}

	return (char *) eeprom_fw_info.ethaddr;
}
#endif	/* Add itaya 2012/02/21 */

/*
 * Boot Infomation EEPROM Read
 */
#if 1	/* Modify itaya 2012/02/21 */
static void * fw_eeprom_read(int bank)
{
	return &eeprom_fw_info;
}
#else	/* Modify itaya 2012/02/21 */
static void * fw_eeprom_read(int bank)
{
	struct boot_fw_eeprom *fw_info = NULL;
	u32 eeplen = sizeof(struct boot_fw_eeprom);

#ifdef CONFIG_CMD_EEPROM
	/* parameter check */
	if ((bank != EEPROM_BANK_0) && (bank != EEPROM_BANK_1)) {
		printf(" Not Supported EEPROM Bank %d\n", bank);
		goto error;
	}

	/* enviroment area malloc */
	fw_info = malloc(eeplen);
	if (!fw_info) {
		printf(" EEPROM Buffer allocation error\n");
		goto error;
	}

	/* eeprom read */
	if (__eeprom_read(CONFIG_SYS_DEF_EEPROM_ADDR, (bank * eeplen),
			(uchar *)fw_info, eeplen) < 0) {
		printf(" EEPROM Bank %d: Read Error(%#x)\n", bank, (bank * eeplen));
		goto error;
	}

	/* magic code check */
	printf("Bank %d - EEPROM Magic Check ....", bank);
#if 1	/* L5 support */
	if ((fw_info->magic & 0xffffff00) != (MAGIC_JHRWL4 & 0xffffff00)) {
		printf("Bad Magic Code(%#x)\n",  fw_info->magic);
		goto error;
	}
#else	/* L5 support */
	if ((fw_info->magic != MAGIC_JHRWL3) && (fw_info->magic != MAGIC_JHRWL4)) {
		printf("Bad Magic Code(%#x)\n",  fw_info->magic);
		goto error;
	}
#endif	/* L5 support */
	printf("OK\n");

	/* crc32 check */
	printf("Bank %d - EEPROM CRC Verify ....", bank);
	if (crc32(0, (uchar *)fw_info, (eeplen-sizeof(fw_info->crc))) != fw_info->crc) {
		printf("Bad CRC(%#x)\n",  fw_info->magic);
		goto error;
	}
	printf("OK\n");

	/* recovery ?? */
	if (bank == EEPROM_BANK_1) {
		if (__eeprom_write(CONFIG_SYS_DEF_EEPROM_ADDR, 0,
					(uchar *)fw_info, eeplen) < 0)
			printf(" EEPROM Bank 0 : recovery error...\n");
	}

	return fw_info;
#endif	/* CONFIG_CMD_EEPROM */

error:
	if (fw_info)
		free(fw_info);
	return NULL;
}
#endif	/* Modify itaya 2012/02/21 */

/*
 * Boot Infomation Update
 */
static int fw_eeprom_bootsts_update(int bank, struct boot_fw_eeprom *fw_info)
{
	int ret = 1;
	u16 update = 0, now = 0;

	if (!fw_info || (bank >= EEPROM_BANK_MAX)) {
		printf("boot status update invalid parameter...\n");
		goto error;
	}

	now = fw_info->bootflags;

	switch (now) {
	case (START_SIDE_A | START_FLAG_OK):				/* 0x0a00 */
	case (START_SIDE_B | START_FLAG_OK):				/* 0x0b00 */
		update = 0;
		break;
	case (START_SIDE_A | START_FLAG_UPDATE):			/* 0x0a01 */
		update = (START_SIDE_A | START_FLAG_FIRST);		/*   -> 0x0a02 */
		break;
	case (START_SIDE_B | START_FLAG_UPDATE):			/* 0x0b01 */
		update = (START_SIDE_B | START_FLAG_FIRST);		/*   -> 0x0b02 */
		break;
	case (START_SIDE_A | START_FLAG_FIRST):				/* 0x0a02 */
		update = (START_SIDE_B | START_FLAG_BACKUP);	/*   -> 0x0b03 */
		break;
	case (START_SIDE_B | START_FLAG_FIRST):				/* 0x0b02 */
		update = (START_SIDE_A | START_FLAG_BACKUP);	/*   -> 0x0a03 */
		break;
	default:											/* 0x**** */
		update = (START_SIDE_A | START_FLAG_FIRST);		/*   -> 0x0a02 */
		break;
	}

	if (update) {
		int b;
		fw_info->bootflags = update;
		fw_info->crc = crc32(0, (uchar *)fw_info,
				(sizeof(*fw_info) - sizeof(fw_info->crc)));

		for (b = 0; b < EEPROM_BANK_MAX; b++) {
			printf("Bank %d : Bootflags update (%#x -> %#x)....",
					b, now, update);
			if (__eeprom_write(CONFIG_SYS_DEF_EEPROM_ADDR,
				eeprom_off_conv(fw_info, &fw_info->bootflags, b),
				(uchar *)&fw_info->bootflags, sizeof(fw_info->bootflags)) < 0) {
				printf("NG\n");
				goto error;
			}
			printf("OK\n");

			printf("Bank %d : CRC update ...", b);
			if (__eeprom_write(CONFIG_SYS_DEF_EEPROM_ADDR,
				eeprom_off_conv(fw_info, &fw_info->crc, b),
				(uchar *)&fw_info->crc, sizeof(fw_info->crc)) < 0) {
				printf("NG\n");
				goto error;
			}
			printf("OK\n");
		}
	}

	ret = 0;
error:
	return ret;
}

/*
 * Kernel Image Load
 */
static int fw_kernel_load(u32 loadaddr, u32 imageaddr)
{
	char nand_cmd[128];

	/* nand command 
	 *  nand read ${loadaddr} ${image offset} size
	 */
	sprintf(nand_cmd, "nand read 0x%x 0x%x 0x800000",
			loadaddr, imageaddr);

	/* nand read is OK -> 1 */
	return ((run_command(nand_cmd, 0) == 1) ? 0 : 1);
}

/*
 * Kernel Image barner
 */
static void fw_version_display(u32 imagaddr, char *version, char *build_ver)
{
	char tmp[2][8];

	if (version) {
		sprintf(tmp[0], "%c%c.%c%c",
			*(version), *(version + 1), *(version + 2), *(version + 3));
	} else {
		sprintf(tmp[0], "NONE");
	}

	if (build_ver) {
		memcpy(tmp[1], build_ver, 4);
		tmp[1][4] = '\0';
	} else {
		sprintf(tmp[1], "NONE");
	}

	printf("+++ Barner +++++++++++++++++++++++++++++++++++\n");
	printf(" Kernel Image Address : %#x\n", imagaddr);
	printf("              Version : %s\n", tmp[0]);
	printf("        Build Version : %s\n", tmp[1]);
	printf("++++++++++++++++++++++++++++++++++++++++++++++\n");
}

#if 0	/* Delete itaya 2012/02/21 */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * TEST Mode Only Function
 */
int do_testmode (struct boot_fw_eeprom *fw_info)
{
#define RESERV_KNL_OFFSET	(0x01A00000)
#if 0
#define RESERV_RFS_VOLNAME	"ubi.mtd=8,1024 root=ubi0:reserve rootfstype=ubifs rw"
#else
#define RESERV_RFS_VOLNAME	"ubi.mtd=8,1024 root=ubi0:reserve rootfstype=ubifs ro"
#endif
#ifndef CONFIG_IPADDR
# define CONFIG_IPADDR		192.168.0.100
#endif
	char * bootmode_args;
	char bootargs[128];
	int err = 0;

	/* Get Bootmode Arguments */
	bootmode_args = getenv("bootmode_args");
	if ((bootmode_args == NULL) || (memcmp(bootmode_args, "test", 4) != 0)) {
		/* Not Test mode */
		goto end;
	}

	/* enviroment setting */
	if (fw_info) {
		fw_setenv_macaddr(fw_info->ethaddr, "ethaddr");		/* env : ethaddr */
	} else {
		fw_setenv_macaddr(MK_STR(CONFIG_ETHADDR), "ethaddr");	/* env : ethaddr */
	}

	/* Test mode bootargs setting */
	memset(bootargs, 0, 128);
	sprintf(bootargs, "console=ttymxc0,115200 %s bootcmd=test ip=%s",
			RESERV_RFS_VOLNAME, MK_STR(CONFIG_IPADDR));
	setenv("bootargs", bootargs);

	printf("+++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
	printf(" Test mode start ...                                   \n");
	printf("  - bootargs : %s\n", bootargs);
	printf("+++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");

	/* Kernel Load */
	fw_kernel_load(CONFIG_SYS_LOAD_ADDR, RESERV_KNL_OFFSET);

	/* Jump Reserved Kernel/RFS */
	run_command("bootm", 0);

	err = 1;
end:	/* end function */
	return err;
}
#endif	/* Delete itaya 2012/02/21 */


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * global functions
 *
 */

/*
 * Nand FlashBoot Command
 *
 */
int do_flboot (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
#define WDOG_TIMEOUT	0xFF

	int rc = 1;
	u16 bootsts;
	int load = 0;
	u32 loadaddr = CONFIG_SYS_LOAD_ADDR;
	u32 imageaddr;
	struct boot_fw_eeprom *fw_info;
	int bank = 0;
#if 1	/* Add itaya 2012/02/21 */
	char *env;
#endif	/* Add itaya 2012/02/21 */

#if 0	/* Delete itaya 2012/02/21 */
retry:	/* bank 0 only retry */
	fw_info = NULL;
#endif	/* Delete itaya 2012/02/21 */

	/* read eeprom */
#if 1	/* Modify itaya 2012/02/21 */
	fw_info = fw_eeprom_read(bank);

	if ((env = getenv("bootflags")) != NULL) {
		fw_info->bootflags = (int)simple_strtol(env, NULL, 16);
	}

	bootsts = fw_info->bootflags;
#else	/* Modify itaya 2012/02/21 */
	for ( ; bank < EEPROM_BANK_MAX; bank++) {
		fw_info = fw_eeprom_read(bank);
		if (fw_info) {
			default_param = 0;
			break;
		}
	}
	if (!default_param) {
		bootsts = fw_info->bootflags;

		PRINTF("%s(%d) : Not Default Parameter(%d) - %#x\n",
			__FUNCTION__, __LINE__, bank, bootsts);
	} else {
		bootsts = 0x0C00;	/* default is C side boot */
		PRINTF("%s(%d) : Default Parameter(%d) - %#x\n",
			__FUNCTION__, __LINE__, bank, bootsts);
	}
#endif	/* Modify itaya 2012/02/21 */

#if 0	/* Delete itaya 2012/02/21 */
	/* test mode checking ... */
	if (do_testmode(fw_info)) {
		return 1;
	}
#endif	/* Delete itaya 2012/02/21 */

	/* Backup status check */
	if ((bootsts & START_STS_MASK) == START_FLAG_BACKUP) {
		printf("Backup start error...\n");
		goto end;
	}

#if 1	/* Add itaya 2012/02/21 */
	/* update */
	if (fw_eeprom_bootsts_update(bank, fw_info)) {
		printf(" Boot Status update error...\n");
	}

	bootsts = fw_info->bootflags;
#endif	/* Add itaya 2012/02/21 */

	/* Boot Side Checking */
	switch (bootsts & START_SIDE_MASK) {
	case START_SIDE_A:
		imageaddr = fw_info->side_a.kernel_addr;
		fw_version_display(fw_info->side_a.kernel_addr,
				(char *)fw_info->side_a.version,
				(char *)fw_info->side_a.build_ver);
		break;
	case START_SIDE_B:
		load = 1;
		imageaddr = fw_info->side_b.kernel_addr;
		fw_version_display(fw_info->side_b.kernel_addr,
				(char *)fw_info->side_b.version,
				(char *)fw_info->side_b.build_ver);
		break;
	default:	/* default parameter */
		imageaddr = 0x00200000;	/* T.B.D */
		fw_version_display(imageaddr, NULL, NULL);
		break;
	}
	PRINTF("%s(%d) : Kernel Image - Nand offset is 0x%08x\n",
		__FUNCTION__, __LINE__, imageaddr);


	/* enviroment setting */
#if 0	/* Delete itaya 2012/02/21 */
	if (!default_param) {
#endif	/* Delete itaya 2012/02/21 */
		fw_setenv_macaddr(fw_info->ethaddr, "ethaddr");		/* env : ethaddr */
#if 0	/* not set env wlanaddr */
		fw_setenv_macaddr(fw_info->wlanaddr, "wlanaddr");	/* env : wlanaddr */
#endif
		fw_setenv_loadaddr(loadaddr);				/* env : loadaddr */
		fw_setenv_rootfs(load);
		fw_setenv_bootargs();					/* env : bootargs */
#if 0	/* Delete itaya 2012/02/21 */
	} else {
		fw_setenv_macaddr((u8 *)MK_STR(CONFIG_ETHADDR), "ethaddr");	/* env : ethaddr */
								/* not set env : wlanaddr */
		fw_setenv_loadaddr(loadaddr);				/* env : loadaddr */
		fw_setenv_rootfs(0);
		fw_setenv_bootargs();					/* env : bootargs */
	}
#endif	/* Delete itaya 2012/02/21 */

	PRINTF("%s(%d) : getenv bootargs - %s\n",
		__FUNCTION__, __LINE__, getenv("bootargs"));

	/* nand read */
	rc = fw_kernel_load(loadaddr, imageaddr);
	if (rc) {
#if 0	/* Delete itaya 2012/02/21 */
		if (bank == EEPROM_BANK_0) {
			bank++;
			goto retry;	/* Retry Bank1 */
		}
#endif	/* Delete itaya 2012/02/21 */
		printf(" Kernel Nand Read Error ...\n");
		goto end;
	}

#if 0	/* Delete itaya 2012/02/21 */
	/* update */
	if (fw_eeprom_bootsts_update(bank, fw_info))
		printf(" Boot Status update error...\n");
#endif	/* Delete itaya 2012/02/21 */

	/* update is Watchdog Start */
	if ((bootsts & START_STS_MASK) != START_FLAG_OK) {
		printf(" Watchdog Timer Start ...\n");
		/* clock enable */
		wdog_clk_enable();

		wdog_config();
		wdog_set_timeout(WDOG_TIMEOUT);
		wdog_enable();
		wdog_ping();
	}

	/* bootm */
	rc = run_command("bootm", 0);
	if (rc) {
#if 0	/* Delete itaya 2012/02/21 */
		if (bank == EEPROM_BANK_0) {
			bank++;
			goto retry;	/* Retry Bank1 */
		}
#endif	/* Delete itaya 2012/02/21 */
		printf(" Kernel Load Error ...\n");
		goto end;
	}
end:
	return 1;
}

/* #define U_BOOT_CMD(name,maxargs,rep,cmd,usage,help) */
U_BOOT_CMD(
	flboot,	1,	1,	do_flboot,
	"Memory image BOOT (main BOOTcommand)",
	""
);

