/*
 * (C) Copyright 2011
 * Working Solar System Production
 *  
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*-----------------------------------------------------------------------
 * LED
 */

#include <common.h>
#include <command.h>
#include <asm/io.h>
#include <asm/errno.h>
#include <asm/arch/mx25.h>
#include <asm/arch/mx25-regs.h>
#include <asm/arch/mx25_pins.h>
#include <asm/arch/iomux.h>
#include <asm/arch/gpio.h>

#ifdef	CMD_LED_DEBUG
# define	PRINTF(fmt,args...)	printf (fmt ,##args)
#else
# define PRINTF(fmt,args...)
#endif

#define LED_NUM	9

struct led_test_t {
	u32	pin;
	u32	val;
};

static struct led_test_t ledlist[] = {
	/* pin			val */
	{ MX25_PIN_CSI_D2,	0x0001 },	/* GPIO1_GPIO[27] */
	{ MX25_PIN_CSI_D3,	0x0002 },	/* GPIO1_GPIO[28] */
	{ MX25_PIN_CSI_D4,	0x0004 },	/* GPIO1_GPIO[29] */
	{ MX25_PIN_CSI_D5,	0x0008 },	/* GPIO1_GPIO[30] */
	{ MX25_PIN_CSI_D6,	0x0010 },	/* GPIO1_GPIO[31] */
	{ MX25_PIN_CSI_D7,	0x0020 },	/* GPIO1_GPIO[6]  */
	{ MX25_PIN_CSI_D8,	0x0040 },	/* GPIO1_GPIO[7]  */
	{ MX25_PIN_CSI_VSYNC,	0x0080 },	/* GPIO1_GPIO[9]  */
	{ MX25_PIN_CSI_HSYNC,	0x0100 } };	/* GPIO1_GPIO[10] */

/*
 * LED H/W Testprogram main function
 *
 * led command is 
 *  > led [control val]
 */
int do_led (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	u32 val;
	int i, j;
	int rcode = 1;

	if (argc < 2) {
		goto cmd_failed;
	}

	val = (simple_strtoul(argv[1], NULL, 16) & 0x01FF);

	/* LED Control */
	for (i = 0; i < LED_NUM; i++) {
		if (val & ledlist[i].val) {
			mxc_set_gpio_dataout(ledlist[i].pin, 1);
			printf ("LED_NUM (0x%04x) : ON\n", ledlist[i].val);
		} else {
			mxc_set_gpio_dataout(ledlist[i].pin, 0);
			printf ("LED_NUM (0x%04x) : OFF\n", ledlist[i].val);
		}
	}

	rcode = 0;
	return rcode;

cmd_failed:
	cmd_usage(cmdtp);
	return rcode;
}

/* #define U_BOOT_CMD(name,maxargs,rep,cmd,usage,help) */
U_BOOT_CMD(
	led,	2,	1,	do_led,
	"LED test command",
	"led [control val]"
);

