/*
 * (C) Copyright 2011
 * Working Solar System Production
 *  
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Speech Function
 */
#include <common.h>
#include <command.h>

#include <spi.h>
#include <imx_spi.h>

#include <asm/arch/mx25.h>
#include <asm/arch/mx25-regs.h>
#include <asm/arch/mx25_pins.h>
#include <asm/arch/iomux.h>
#include <asm/arch/gpio.h>

/*-----------------------------------------------------------------------
 * Speech
 */
#ifdef	CMD_SPEECH_DEBUG
# define PRINTF(fmt,args...)	printf (fmt ,##args)
#else
# define PRINTF(fmt,args...)
#endif

#ifdef CONFIG_DEFAULT_SPI_BUS
# define SPI_DEFBUS	CONFIG_DEFAULT_SPI_BUS
#else
# define SPI_DEFBUS	0
#endif	/* CONFIG_DEFAULT_SPI_BUS */

#ifdef CONFIG_SYS_SPI_SPEECH_DEVID
# define SPI_DEVICEID	CONFIG_SYS_SPI_SPEECH_DEVID
#else
# define SPI_DEVICEID	0
#endif	/* CONFIG_SYS_SPI_SPEECH_DEVID */

#ifdef CONFIG_SYS_SPI_MAXHZ
# define SPI_MAXHZ	CONFIG_SYS_SPI_MAXHZ
#else
# define SPI_MAXHZ	25000000
#endif	/* CONFIG_SYS_SPI_MAXHZ */

#ifdef CONFIG_SYS_SPI_MODE
# define SPI_DEFMODE	CONFIG_SYS_SPI_MODE
#else
# define SPI_DEFMODE	0
#endif	/* CONFIG_SYS_SPI_MODE */

#ifndef MAX_SPI_BYTES
# define MAX_SPI_BYTES 32	/* Maximum number of bytes we can handle */
#endif

/*
 * Values from last command.
 */
static u32		device;
static int		bitlen;
static uchar 		dout[MAX_SPI_BYTES];
static uchar 		din[MAX_SPI_BYTES];
static struct spi_slave *pslave = NULL;

#define ALLOW_CBUSY_NOT_CONNECT
#define ML22823_DEBUG

#define MAX_CBUSY_DELAY		(2)				/* 2 usec */
#define MAX_CBUSY_WAIT		(200000)		/* 200 msec */
#define MAX_CBUSY_COUNT		(MAX_CBUSY_WAIT / MAX_CBUSY_DELAY)

#define MAX_NCR_DELAY		(2)				/* 2 usec */
#define MAX_NCR_WAIT		(200000)		/* 200 msec */
#define MAX_NCR_COUNT		(MAX_NCR_WAIT / MAX_NCR_DELAY)

#define CMD_PUP			0x00
	#define PUP_S1			0x02
	#define PUP_S0			0x01

	#define PUP_BANK_1		0x00
	#define PUP_BANK_2		0x01
	#define PUP_BANK_4		0x02

#define CMD_PDWN		0x20
#define CMD_RDSTAT		0xB0
	#define RDSTAT_BUSYB1	0x20
	#define RDSTAT_BUSYB0	0x10
	#define RDSTAT_NCR1		0x02
	#define RDSTAT_NCR0		0x01
#define CMD_AMODE		0x04
	#define AMODE_FAD		0x80
	#define AMODE_DAG1		0x40
	#define AMODE_DAG0		0x20
	#define AMODE_AIG1		0x10
	#define AMODE_AIG0		0x08
	#define AMODE_DAEN		0x04
	#define AMODE_SPEN		0x02
	#define AMODE_PUP		0x01
	#define AMODE_DEFAULT	(AMODE_FAD  | AMODE_DAG1 | AMODE_DAEN | AMODE_SPEN | AMODE_PUP)
							/* Fade-out processing: Yes */
							/* Gain of the internal DAC signal: Input ON (0 dB) */
							/* Gain of an analog signal from the AIN pin: Input ON (0 dB) */
							/* DAC section: Power-up state */
							/* speaker section: Power-up state */
							/* Pop noise suppression: Yes */
	#define AMODE_ECOCUTE	(AMODE_FAD  | AMODE_AIG1 | AMODE_SPEN | AMODE_PUP)

#define CMD_PLAY		0x40
	#define PLAY_CH			0x01

#define CMD_STOP		0x60
	#define STOP_CH1			0x02
	#define STOP_CH0			0x01

#define CMD_FADR		0x30
	#define FADR_CH			0x01

#define CMD_START		0x50
	#define START_CH1			0x02
	#define START_CH0			0x01

#define CMD_MUON		0x70
#define CMD_SLOOP		0x80
	#define SLOOP_CH1			0x02
	#define SLOOP_CH0			0x01
#define CMD_CLOOP		0x90
	#define CLOOP_CH1			0x02
	#define CLOOP_CH0			0x01
#define CMD_CVOL		0xA0
	#define CVOL_CH1			0x02
	#define CVOL_CH0			0x01
	#define CVOL_MASK			0x1F
#define CMD_AVOL		0x08
	#define AVOL_MASK			0x3F

#define	ML22823_SEL0		(MX25_PIN_CSPI1_SS1)		/* Output */
#define	ML22823_SEL1		(MX25_PIN_CSPI1_RDY)		/* Output */
#define	ML22823_CBUSY		(MX25_PIN_UART1_RTS)		/* Input */
#define	ML22823_RESETB		(MX25_PIN_UART2_RTS)		/* Output */

static unsigned char analog_volume = 0x30;
static unsigned char channel_volume = 0x00;

static int sound_loop    = 0;
#ifdef ALLOW_CBUSY_NOT_CONNECT
static int cbusy_connect = 0;
#endif /* ALLOW_CBUSY_NOT_CONNECT */

#define msleep(ms)			udelay(ms * 1000)

/*
 * ml22825_reset
 *  - ml22825 H/W reset
 */
static void ml22825_reset(void)
{
#ifdef ALLOW_CBUSY_NOT_CONNECT
	int val;
#endif /* ALLOW_CBUSY_NOT_CONNECT */

#ifdef ML22823_DEBUG
	printf("ML22823 Reset\n");
#endif	/* ML22823_DEBUG */

	/* memory bank slways select 0 */
	mxc_set_gpio_dataout(ML22823_SEL0, 0);
	mxc_set_gpio_dataout(ML22823_SEL1, 0);

	mxc_set_gpio_dataout(ML22823_RESETB, 0);
	/* RESETB input pulse width : min 100usec */
	msleep(1);

	mxc_set_gpio_dataout(ML22823_RESETB, 1);
	msleep(2);

#ifdef ALLOW_CBUSY_NOT_CONNECT
	cbusy_connect = 0;
	val = mxc_gpio_get(ML22823_CBUSY);
	if (val == 1) {
#ifdef ML22823_DEBUG
		printf("ML22823 CBUSY Connect\n");
#endif	/* ML22823_DEBUG */
		cbusy_connect = 1;
	} else {
#ifdef ML22823_DEBUG
		printf("ML22823 CBUSY Not Connect\n");
#endif	/* ML22823_DEBUG */
	}
#endif /* ALLOW_CBUSY_NOT_CONNECT */
}

static int ml22825_wait_ready (void)
{
	int i;
	int val = 0;

#ifdef ALLOW_CBUSY_NOT_CONNECT
	if (!cbusy_connect) {
		udelay(2);

		return 0;
	}
#endif /* ALLOW_CBUSY_NOT_CONNECT */

	for (i = 0; i < MAX_CBUSY_COUNT; i++) {
		val = mxc_gpio_get(ML22823_CBUSY);
		if (val == 1) {
#ifdef ML22823_DEBUG
			printf("ML22823_CBUSY = %d\n", val);
#endif	/* ML22823_DEBUG */

			return 0;
		}
		udelay(MAX_CBUSY_DELAY);
	}
#ifdef ML22823_DEBUG
	printf("ML22823_CBUSY = %d\n", val);
#endif	/* ML22823_DEBUG */

	printf("Error: ML22823 ready wait timeout\n");

	return -1;
}

static int ml22825_cmd_write (struct spi_slave *slave, unsigned char cmd)
{
#ifdef ML22823_DEBUG
	printf("ML22823 cmd execute %02x\n", cmd);
#endif	/* ML22823_DEBUG */

	if (ml22825_wait_ready () < 0) {
		/* device busy */
		return -1;
	}

	dout[0] = cmd;
	bitlen  = 8;

	if(spi_xfer(slave, bitlen, dout, 0,
				SPI_XFER_BEGIN | SPI_XFER_END) != 0) {
		printf("Error with the SPI transaction.\n");
		return -1;
	}

	return 0;
}

static int ml22825_cmd_write_read (struct spi_slave *slave, unsigned char cmd, unsigned char *result)
{
	unsigned char cmd0 = 0;

#ifdef ML22823_DEBUG
	printf("ML22823 cmd execute %02x\n", cmd);
#endif	/* ML22823_DEBUG */

	if (ml22825_wait_ready () < 0) {
		/* device busy */
		return -1;
	}

	dout[0] = cmd;
	bitlen  = 8;

	if(spi_xfer(slave, bitlen, dout, 0,
				SPI_XFER_BEGIN | SPI_XFER_END) != 0) {
		printf("Error with the SPI transaction.\n");
		return -1;
	}

	if (ml22825_wait_ready () < 0) {
		/* device busy */
		return -1;
	}

	dout[0] = cmd0;
	bitlen  = 8;

	if(spi_xfer(slave, bitlen, dout, din,
				SPI_XFER_BEGIN | SPI_XFER_END) != 0) {
		printf("Error with the SPI transaction.\n");
		return -1;
	}

	if (result) {
		*result = din[0];
	}

	return 0;
}

/*
 * ml22825_pup
 *  - pup command
 */
static int ml22825_pup (struct spi_slave *slave, unsigned char bank)
{
	int err;
	unsigned char cmd;

#ifdef ML22823_DEBUG
	printf("ML22823 CMD_PUP bank=%d\n", bank);
#endif	/* ML22823_DEBUG */

	/*
	 *	command	0	0	0	0	0	0	S1	S0
	 */

	cmd = (CMD_PUP | (bank & (PUP_S1 | PUP_S0)));

	spi_claim_bus(slave);
	err = ml22825_cmd_write (slave, cmd);
	spi_release_bus(slave);

	if (err) {
		printf("Error: ML22823 CMD_PUP failure err=%d\n", err);
		return err;
	}

	/* wakeup wait */
	msleep(10);

	return 0;
}

/*
 * ml22825_pdwn
 *  - pdwn command
 */
static int ml22825_pdwn (struct spi_slave *slave)
{
	int err;

#ifdef ML22823_DEBUG
	printf("ML22823 CMD_PDWN\n");
#endif	/* ML22823_DEBUG */

	/*
	 *	command	0	0	1	0	0	0	0	0
	 */

	spi_claim_bus(slave);
	err = ml22825_cmd_write (slave, CMD_PDWN);
	spi_release_bus(slave);

	if (err < 0) {
		printf("Error: ML22823 CMD_PDWN failure err=%d\n", err);
		return err;
	}

	return 0;
}

/*
 * ml22825_rdstat
 *  - rdstat command
 */
static int ml22825_rdstat (struct spi_slave *slave, unsigned char *pstat)
{
	int err;
	unsigned char rdstat = 0;

#ifdef ML22823_DEBUG
	printf("ML22823 RDSTAT\n");
#endif	/* ML22823_DEBUG */

	/*
	 *	command	1	0	1	1	0	0	0	0
	 */

	spi_claim_bus(slave);
	err = ml22825_cmd_write_read (slave, CMD_RDSTAT, &rdstat);
	spi_release_bus(slave);

	if (err < 0) {
		printf("Error: ML22823 CMD_RDSTAT failure err=%d\n", err);
		return err;
	}

	if (pstat) {
		*pstat = rdstat;
	}

#ifdef ML22823_DEBUG
	printf("ML22823 CMD_RDSTAT (rdstat=%02x)\n", rdstat);
#endif	/* ML22823_DEBUG */

	return 0;
}

/*
 * ml22825_amode
 *  - amode command
 */
static int ml22825_amode (struct spi_slave *slave, unsigned char amode)
{
	int err;

#ifdef ML22823_DEBUG
	printf("ML22823 CMD_AMODE\n");
#endif	/* ML22823_DEBUG */

	/*
	 *	command		0	0		0		0		0		1		0		0	Byte 1
	 *				FAD	DAG1	DAG0	AIG1	AIG0	DAEN	SPEN	PUP	Byte 2
	 *				1	1		0		1		0		1		1		1
	 */

	spi_claim_bus(slave);
	err = ml22825_cmd_write (slave, CMD_AMODE);

	if (err < 0) {
		printf("Error: ML22823 CMD_AMODE failure err=%d\n", err);
		spi_release_bus(slave);
		return err;
	}
	err = ml22825_cmd_write (slave, amode);
	spi_release_bus(slave);

	if (err < 0) {
		printf("Error: ML22823 CMD_AMODE failure err=%d\n", err);
		return err;
	}

#ifdef ALLOW_CBUSY_NOT_CONNECT
	if (!cbusy_connect) {
		msleep(200);
	}
#endif /* ALLOW_CBUSY_NOT_CONNECT */

	return 0;
}

/*
 * ml22825_play
 *  - play command
 */
static int ml22825_play (struct spi_slave *slave, int ch, unsigned short fadr)
{
	int err;
	int i;
	unsigned char cmd;
	unsigned char rdstat = 0;
	unsigned char ncr_signal;

#ifdef ML22823_DEBUG
	printf("ML22823 CMD_PLAY ch=%d fadr=%04x\n", ch, fadr);
#endif	/* ML22823_DEBUG */

	/* Wait NCR signal High */
	ncr_signal = (1 << (ch & 0x01));
	for (i = 0; i < MAX_NCR_COUNT; i++) {
		err = ml22825_rdstat(slave, &rdstat);
		if (rdstat & ncr_signal) {
			break;
		}
		udelay(MAX_NCR_DELAY);
	}
	if (i == MAX_NCR_COUNT) {
		printf("Error: ML22823 NCR wait timeout\n");

		return -1;
	}

	/*
	 *	command	0	1	0	0	F9	F8	0	CH	byte1
	 *			F7	F6	F5	F4	F3	F2	F1	F0	byte2
	 */

	cmd = (CMD_PLAY | ((fadr & 0x0300) >> 6) | (ch & PLAY_CH));

	spi_claim_bus(slave);
	err = ml22825_cmd_write (slave, cmd);
	if (err < 0) {
		printf("Error: ML22823 CMD_PLAY failure err=%d\n", err);
		spi_release_bus(slave);
		return err;
	}

	cmd = (fadr & 0xff);
	err = ml22825_cmd_write (slave, cmd);
	spi_release_bus(slave);

	if (err < 0) {
		printf("Error: ML22823 CMD_PLAY failure err=%d\n", err);
		return err;
	}

	return 0;
}

/*
 * ml22825_stop
 *  - stop command
 */
static int ml22825_stop (struct spi_slave *slave, unsigned char ch_mask)
{
	int err;
	unsigned char cmd;

#ifdef ML22823_DEBUG
	printf("ML22823 CMD_STOP ch_mask=%d\n", ch_mask);
#endif	/* ML22823_DEBUG */

	/*
	 *	command	0	1	1	0	0	0	CH1	CH0
	 */

	cmd = (CMD_STOP | (ch_mask & (STOP_CH1 | STOP_CH0)));

	spi_claim_bus(slave);
	err = ml22825_cmd_write (slave, cmd);
	spi_release_bus(slave);

	if (err < 0) {
		printf("Error: ML22823 CMD_STOP failure err=%d\n", err);
		return err;
	}

	return 0;
}

/*
 * ml22825_sloop
 *  - sloop command
 */
static int ml22825_sloop (struct spi_slave *slave, unsigned char ch_mask)
{
	int err;
	int i;
	int ch;
	unsigned char cmd;
	unsigned char rdstat;
	unsigned char ncr_signal;

#ifdef ML22823_DEBUG
	printf("ML22823 CMD_SLOOP ch_mask=%d\n", ch_mask);
#endif	/* ML22823_DEBUG */

	for (ch = 0; ch < 2; ch++) {
		if (!(ch_mask & (1 << ch))) {
			continue;
		}

		/* Wait NCR signal High */
		ncr_signal = (1 << ch);
		for (i = 0; i < MAX_NCR_COUNT; i++) {
			err = ml22825_rdstat(slave, &rdstat);
			if (rdstat & ncr_signal) {
				break;
			}
			udelay(MAX_NCR_DELAY);
		}
		if (i == MAX_NCR_COUNT) {
			printf("Error: ML22823 NCR wait timeout\n");

			return -1;
		}
	}

	/*
	 *	command	1	0	0	0	0	0	CH1	CH0			
	 */

	cmd = (CMD_SLOOP | (ch_mask & (SLOOP_CH1 | SLOOP_CH0)));

	spi_claim_bus(slave);
	err = ml22825_cmd_write (slave, cmd);
	spi_release_bus(slave);

	if (err < 0) {
		printf("Error: ML22823 CMD_SLOOP failure err=%d\n", err);
		return err;
	}

	return 0;
}

/*
 * ml22825_cloop
 *  - cloop command
 */
static int ml22825_cloop (struct spi_slave *slave, unsigned char ch_mask)
{
	int err;
	unsigned char cmd;

#ifdef ML22823_DEBUG
	printf("ML22823 CMD_CLOOP ch_mask=%d\n", ch_mask);
#endif	/* ML22823_DEBUG */

	/*
	 *	command	1	0	0	1	0	0	CH1	CH0	
	 */

	cmd = (CMD_CLOOP | (ch_mask & (CLOOP_CH1 | CLOOP_CH0)));

	spi_claim_bus(slave);
	err = ml22825_cmd_write (slave, cmd);
	spi_release_bus(slave);

	if (err < 0) {
		printf("Error: ML22823 CMD_CLOOP failure err=%d\n", err);
		return err;
	}

	return 0;
}


/*
 * ml22825_cvol
 *  - cvol command
 */
static int ml22825_cvol (struct spi_slave *slave, unsigned char ch_mask, unsigned char cvol)
{
	int err;
	unsigned char cmd;

#ifdef ML22823_DEBUG
	printf("ML22823 CMD_CVOL ch_mask=%d cvol=%04x\n", ch_mask, cvol);
#endif	/* ML22823_DEBUG */

	/*
	 *	command	1	0	1	0	0	0	CH1	CH0	Byte1
	 *			0	0	0	CV4	CV3	CV2	CV1	CV0	Byte2
	 */

	cmd = (CMD_CVOL | (ch_mask & (CVOL_CH1 | CVOL_CH0)));

	spi_claim_bus(slave);
	err = ml22825_cmd_write (slave, cmd);
	if (err < 0) {
		printf("Error: ML22823 CMD_CVOL failure err=%d\n", err);
		spi_release_bus(slave);
		return err;
	}

	cmd = (cvol & CVOL_MASK);
	err = ml22825_cmd_write (slave, cmd);
	spi_release_bus(slave);

	if (err < 0) {
		printf("Error: ML22823 CMD_CVOL failure err=%d\n", err);
		return err;
	}

#ifdef ALLOW_CBUSY_NOT_CONNECT
	if (!cbusy_connect) {
		msleep(10);
	}
#endif /* ALLOW_CBUSY_NOT_CONNECT */

	return 0;
}


/*
 * ml22825_avol
 *  - avol command
 */
static int ml22825_avol (struct spi_slave *slave, unsigned char avol)
{
	int err;
	unsigned char cmd;

#ifdef ML22823_DEBUG
	printf("ML22823 CMD_AVOL avol=%04x\n", avol);
#endif	/* ML22823_DEBUG */

	/*
	 *	command	0	0	0	0	1	0	0	0	Byte1
	 *			0	0	AV5	AV4	AV3	AV2	AV1	AV0	Byte2
	 */

	spi_claim_bus(slave);
	err = ml22825_cmd_write (slave, CMD_AVOL);

	if (err < 0) {
		printf("Error: ML22823 CMD_AVOL failure err=%d\n", err);
		spi_release_bus(slave);
		return err;
	}

	cmd = (avol & AVOL_MASK);
	err = ml22825_cmd_write (slave, cmd);
	spi_release_bus(slave);

	if (err < 0) {
		printf("Error: ML22823 CMD_AVOL failure err=%d\n", err);
		return err;
	}

#ifdef ALLOW_CBUSY_NOT_CONNECT
	if (!cbusy_connect) {
		msleep(10);
	}
#endif /* ALLOW_CBUSY_NOT_CONNECT */

	return 0;
}

/*
 * ml22823_init
 *  - ml22823 init
 */
static int ml22823_init (struct spi_slave *slave, unsigned char amode)
{
	int err;

	if ((err = ml22825_amode (slave, amode)) ||
		(err = ml22825_avol(slave, analog_volume)) ||
		(err = ml22825_cvol(slave, CVOL_CH0, 0))) {
		printf("Error: ML22823 init fail\n");
		return err;
	}

	return 0;
}

/*
 * speech_pup
 *  - power up
 */
static int speech_pup (struct spi_slave *slave)
{
	int err;

	if ((err = ml22825_pup (slave, PUP_BANK_1))) {
		return err;
	}
	if ((err = ml22823_init (slave, AMODE_DEFAULT))) {
		return err;
	}

	return 0;
}

/*
 * speech_play
 *  - play
 */
static int speech_play (struct spi_slave *slave, int fadr, int loop)
{
	int err;

	if ((err = ml22825_play (slave, 0, (unsigned short) fadr))) {
		return err;
	}

	sound_loop = 0;
	if (loop) {
		if ((err = ml22825_sloop(slave, SLOOP_CH0))) {
			return err;
		}
		sound_loop = 1;
	}

	return 0;
}

/*
 * speech_stop
 *  - play stop
 */
static int speech_stop (struct spi_slave *slave)
{
	int err;

	if (sound_loop) {
		ml22825_cloop(slave, SLOOP_CH0);
		sound_loop = 0;
	}

	if ((err = ml22825_stop(slave, STOP_CH0))) {
		return err;
	}

	return 0;
}

/*
 * speech_cvol
 *  - channel volume
 */
static int speech_cvol (struct spi_slave *slave,  int cvol)
{
	int err;

	channel_volume = (unsigned char) cvol;

	if ((err = ml22825_cvol(slave, CVOL_CH0, channel_volume))) {
		return err;
	}

	return 0;
}

/*
 * speech_avol
 *  - analog volume
 */
static int speech_avol (struct spi_slave *slave,  int avol)
{
	int err;

	analog_volume = (unsigned char) avol;

	if ((err = ml22825_avol(slave, analog_volume))) {
		return err;
	}

	return 0;
}

/*
 * speech_pdwn
 *  - power down
 */
static int speech_pdwn (struct spi_slave *slave)
{
	int err;

	if (sound_loop) {
		ml22825_cloop(slave, SLOOP_CH0);
		sound_loop = 0;
	}

	ml22825_stop (slave, STOP_CH0);

	if ((err = ml22825_pdwn(slave))) {
		return err;
	}

	return 0;
}

/*
 * speech_ecocute
 *  - ecocute on/off
 */
static int speech_ecocute (struct spi_slave *slave, int on)
{
	int err;
	unsigned char amode = AMODE_DEFAULT;

	ml22825_reset ();

	if ((err = ml22825_pup (slave, PUP_BANK_1))) {
		return err;
	}

	if (on) {
		amode = AMODE_ECOCUTE;
	}
	if ((err = ml22823_init (slave, amode))) {
		return err;
	}

	return 0;
}

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Speech Test sequense
 */
#ifdef CONFIG_JHRWL3_DEBUG
static void speech_test_sequense(struct spi_slave *slave)
{
	printf("++++++ CSPI TEST SEQUENSE ++++++\n");

	ml22825_reset ();

	speech_pup(slave);

	speech_play(slave, 0, 0);

	msleep(5000);

	speech_pdwn(slave);

}
#endif	/* CONFIG_JHRWL3_DEBUG */

/****
 * speech command main
 ****/
int do_speech (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	u32 val;
	int rcode = 1;

	if (argc < 2)
		goto cmd_failed;

	if (!pslave)
		pslave = spi_setup_slave(SPI_DEFBUS, SPI_DEVICEID,
				SPI_MAXHZ, SPI_DEFMODE);
	if (!pslave) {
		printf("not spi devcie ...\n");
		goto err_setup_failed;
	}

	if (!memcmp(argv[1], "pup", 3)) {
		/* Power UP */
		speech_pup(pslave);

	} else if (!memcmp(argv[1], "pdwn", 4)) {
		/* Power Down */
		speech_pdwn(pslave);

	} else if (!memcmp(argv[1], "play", 4)) {
		int loop = 0;

		if (argc < 3)
			goto cmd_failed;

		/* play */
		val = simple_strtoul(argv[2], NULL, 10);
		if (val >= 1024) {	/* phrase address */
			printf("phrase address is 0 - 1023\n");
			goto cmd_failed;
		}

		if (argc > 3)
			loop = simple_strtoul(argv[3], NULL, 10);

		speech_play(pslave, val, loop);

	} else if (!memcmp(argv[1], "stop", 4)) {
		/* stop */
		speech_stop(pslave);

	} else if (!memcmp(argv[1], "avol", 4)) {
		if (argc < 3)
			goto cmd_failed;
		/* analog volume */
		val = simple_strtoul(argv[2], NULL, 16);
		if (val & ~AVOL_MASK) {	/* analog volume */
			printf("analog volume is 0x00 - 0x%2X\n", AVOL_MASK);
			goto cmd_failed;
		}
		speech_avol(pslave, val);
	} else if (!memcmp(argv[1], "cvol", 4)) {
		if (argc < 3)
			goto cmd_failed;
		/* channel volume */
		val = simple_strtoul(argv[2], NULL, 16);
		if (val & ~CVOL_MASK) {	/* channel volume */
			printf("channel volume is 0x00 - 0x%2X\n", CVOL_MASK);
			goto cmd_failed;
		}
		speech_cvol(pslave, val);
	} else if (!memcmp(argv[1], "amode", 5)) {
		if (argc < 3)
			goto cmd_failed;
		/* amode param */
		val = simple_strtoul(argv[2], NULL, 16);
		ml22825_amode(pslave, val);
	} else if (!memcmp(argv[1], "reset", 5)) {
		/* Reset */
		ml22825_reset();
	} else if (!memcmp(argv[1], "ecocute", 7)) {
		int on = 0;
		if (argc < 3)
			goto cmd_failed;
		if (!memcmp(argv[2], "on", 2)) {
			on = 1;
		} else if (!memcmp(argv[2], "off", 3)) {
			on = 0;
		} else {
			printf("on or off\n");
			goto cmd_failed;
		}

		speech_ecocute(pslave, on);
#ifdef CONFIG_JHRWL3_DEBUG
	} else if (!memcmp(argv[1], "test", 4)) {
		speech_test_sequense(pslave);
#endif	/* CONFIG_JHRWL3_DEBUG */
	} else {
		goto cmd_failed;
	}

	rcode = 0;
	goto end;

cmd_failed:
	/* Command Faild Function */
	/* spi_free_slave(pslave);
	pslave = NULL; */
err_setup_failed:
	cmd_usage(cmdtp);

end:
	return rcode;
}

/* #define U_BOOT_CMD(name,maxargs,rep,cmd,usage,help) */
U_BOOT_CMD(
	speech,	4,	1,	do_speech,
	"SPEECH test command",
	"pup    : power up\n"
	"       pdwn   : power down\n"
	"       play   : play\n"
	"       stop   : play stop\n"
	"       avol   : analog volume\n"
	"       cvol   : channel volume\n"
	"       amode  : amode\n"
	"       reset  : H/W reset\n"
	"       ecocute: ecocute\n"
);


