/*
 * (C) Copyright 2011
 * Working Solar System Production
 *  
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*-----------------------------------------------------------------------
 * SWITCH
 */

#include <common.h>
#include <command.h>
#include <asm/io.h>
#include <asm/errno.h>
#include <asm/arch/mx25.h>
#include <asm/arch/mx25-regs.h>
#include <asm/arch/mx25_pins.h>
#include <asm/arch/iomux.h>
#include <asm/arch/gpio.h>

//#define CONFIG_LED_CHANGE	/* LED Setting Configuration */
#undef CONFIG_LED_CHANGE

#define CMD_SWITCH_DEBUG
#ifdef	CMD_SWITCH_DEBUG
# define PRINTF(fmt,args...)	printf (fmt ,##args)
# define mdelay(ms)		udelay(ms * 1000)
#else
# define PRINTF(fmt,args...)
# define mdelay(ms)
#endif

#ifdef CONFIG_LED_CHANGE
static void led_control(iomux_pin_name_t pin, u8 val)
{
	mxc_set_gpio_dataout(pin, (val & 1));
}

static void led_initialize(void)
{
	led_control(MX25_PIN_CSI_D2, 0);
	led_control(MX25_PIN_CSI_D3, 0);
	led_control(MX25_PIN_CSI_D4, 0);
	led_control(MX25_PIN_CSI_D5, 0);
	led_control(MX25_PIN_CSI_D6, 0);
	led_control(MX25_PIN_CSI_D7, 0);
	led_control(MX25_PIN_CSI_D8, 0);
	led_control(MX25_PIN_CSI_VSYNC, 0);
	led_control(MX25_PIN_CSI_HSYNC, 0);
}

#define LED_CONTROL(pin, val)	led_control(pin, val)
#define LED_INITIALIZE()	led_initialize()
#else
#define LED_CONTROL(pin, val)
#define LED_INITIALIZE()
#endif	/* CONFIG_LED_CHANGE */

#define STATUS_INFO	"Switch Name          | Status\n"
#define STATUS_BAR	"-----------------------------\n"

#define HOME_SWITCH_GPIO	(11)
# define HOME_SWITCH_ON_CONSOLE		"Home Switch           [ ON  ]"
# define HOME_SWITCH_OFF_CONSOLE	"Home Switch           [ OFF ]"
# define HOME_SWITCH_LED_PIN		MX25_PIN_CSI_D2

#define BATH_SWITCH_GPIO	(10)
# define BATH_SWITCH_ON_CONSOLE		"Bath Automatic Switch [ ON  ]"
# define BATH_SWITCH_OFF_CONSOLE	"Bath Automatic Switch [ OFF ]"
# define BATH_SWITCH_LED_PIN		MX25_PIN_CSI_D3

#define DRIVING_SWITCH_GPIO	(9)
# define DRIVE_SWITCH_ON_CONSOLE	"Driving Change Switch [ ON  ]"
# define DRIVE_SWITCH_OFF_CONSOLE	"Driving Change Switch [ OFF ]"
# define DRIVE_SWITCH_LED_PIN		MX25_PIN_CSI_D4

#define RESET_B			(0)

#define SWITCH_HIGH		(1)
#define SWITCH_LOW		(0)

/*
 * Switch H/W Testprogram main function
 *
 * switchcheck 
 */
int do_switchcheck (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	u32 home, bath, drive;
	u32 val;

	home = bath = drive = 0;
	LED_INITIALIZE();

	printf("++++++++++++ Switch check Test command ++++++++++++++\n");
	printf(" end     ) \"CTRL + C\" Input ...\n");
	printf(" console ) Pushed Switched name display\n");
	printf("+++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
	printf("%s", STATUS_BAR);
	printf("%s", STATUS_INFO);
	printf("%s", STATUS_BAR);

	for (;;) {
		/* CTRL + C Check */
		if (ctrlc()) {
			putc ('\n');
			break;
		}

		/* Home Switch Check */
		val = mxc_gpio_get(MX25_PIN_D9);
		if ((val == SWITCH_LOW) && (home == 0)) {
			printf("%s\n", HOME_SWITCH_ON_CONSOLE);
			LED_CONTROL(HOME_SWITCH_LED_PIN, 1);
			home = 1;
		} else if ((val == SWITCH_HIGH) && (home == 1)) {
			printf("%s\n", HOME_SWITCH_OFF_CONSOLE);
			LED_CONTROL(HOME_SWITCH_LED_PIN, 0);
			home = 0;
		}

		/* Bath Automatic Switch Check */
		val = mxc_gpio_get(MX25_PIN_D10);
		if ((val == SWITCH_LOW) && (bath == 0)) {
			printf("%s\n", BATH_SWITCH_ON_CONSOLE);
			LED_CONTROL(BATH_SWITCH_LED_PIN, 1);
			bath = 1;
		} else if ((val == SWITCH_HIGH) && (bath == 1)) {
			printf("%s\n", BATH_SWITCH_OFF_CONSOLE);
			LED_CONTROL(BATH_SWITCH_LED_PIN, 0);
			bath = 0;
		}

		/* Driving Change Switch Check */
		val = mxc_gpio_get(MX25_PIN_D11);
		if ((val == SWITCH_LOW) && (drive == 0)) {
			printf("%s\n", DRIVE_SWITCH_ON_CONSOLE);
			LED_CONTROL(DRIVE_SWITCH_LED_PIN, 1);
			drive = 1;
		} else if ((val == SWITCH_HIGH) && (drive == 1)) {
			printf("%s\n", DRIVE_SWITCH_OFF_CONSOLE);
			LED_CONTROL(DRIVE_SWITCH_LED_PIN, 0);
			drive = 0;
		}
	}

	LED_INITIALIZE();

	return 0;
}

/* #define U_BOOT_CMD(name,maxargs,rep,cmd,usage,help) */
U_BOOT_CMD(
	switchcheck,	1,	1,	do_switchcheck,
	"Switch test command",
	"switchcheck .. end is \"CTRL + C\""
);

