/*
 * (C) Copyright 2011
 * Working Solar System Production
 *  
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*-----------------------------------------------------------------------
 * Voice
 */

#include <common.h>
#include <command.h>
#include <asm/io.h>
#include <asm/errno.h>
#include <asm/arch/mx25.h>
#include <asm/arch/mx25-regs.h>
#include <asm/arch/mx25_pins.h>
#include <asm/arch/iomux.h>
#include <asm/arch/gpio.h>

#ifdef	CMD_VOICE_DEBUG
# define PRINTF(fmt,args...)	printf (fmt ,##args)
#else
# define PRINTF(fmt,args...)
#endif

static u8 speeker = 1;
static u8 microphone = 1;
static u8 speeker_vol = 0;
static u8 talk = 0;

const char speeker_str[][8] = {"[ ON ]", "[ OFF ]"};
const char mic_str[][8] = {"[ ON ]", "[ OFF ]"};
const char vol_str[][8] = {"[ BIG ]", "[ MID ]", "[ ??? ]", "[ SML ]"};
const char talk_str[][8] = {"[ OFF ]", "[ ON ]"};

static void now_status(void)
{
	printf("+++++ Status GPIO +++++\n");
	printf(" - Speeker     : %s\n", speeker_str[speeker]);
	printf(" - Microphone  : %s\n", mic_str[microphone]);
	printf(" - Speeker Vol : %s\n", vol_str[speeker_vol]);
	printf(" - Talk In     : %s\n", talk_str[talk]);
};

/****
 * voice command main
 ****/
int do_voice (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int rcode = 1;

	if (argc < 2)
		goto cmd_failed;

	switch(simple_strtoul(argv[1], NULL, 10)) {
	case 1:		/* Speeker on */
		mxc_set_gpio_dataout(MX25_PIN_KPP_ROW3, 0);	/* Low */
		speeker = 0;
		break;
	case 2:		/* Speeker off */
		mxc_set_gpio_dataout(MX25_PIN_KPP_ROW3, 1);	/* High */
		speeker = 1;
		break;
	case 3:		/* microphone on */
		mxc_set_gpio_dataout(MX25_PIN_KPP_ROW2, 0);	/* Low */
		microphone = 0;
		break;
	case 4:		/* microphone off */
		mxc_set_gpio_dataout(MX25_PIN_KPP_ROW2, 1);	/* High */
		microphone = 1;
		break;
	case 5:		/* speeker volume small */
		mxc_set_gpio_dataout(MX25_PIN_KPP_COL0, 1);
		mxc_set_gpio_dataout(MX25_PIN_KPP_COL1, 1);	/* High - High */
		speeker_vol = 3;
		break;
	case 6:		/* speeker volume middle */
		mxc_set_gpio_dataout(MX25_PIN_KPP_COL0, 0);
		mxc_set_gpio_dataout(MX25_PIN_KPP_COL1, 1);	/* Low - High */
		speeker_vol = 1;
		break;
	case 7:		/* speeker volume big */
		mxc_set_gpio_dataout(MX25_PIN_KPP_COL0, 0);
		mxc_set_gpio_dataout(MX25_PIN_KPP_COL1, 0);	/* Low - Low */
		speeker_vol = 0;
		break;
	case 8:		/* mute on */
	case 9:		/* mute off */
		printf("mute not supported\n");
		goto cmd_failed;
	case 10:	/* receive output on */
		mxc_set_gpio_dataout(MX25_PIN_KPP_COL3, 1);	/* High */
		talk = 1;
		break;
	case 11:	/* receive outout off */
		mxc_set_gpio_dataout(MX25_PIN_KPP_COL3, 0);	/* Low */
		talk = 0;
		break;
	default:
		goto cmd_failed;
	}
	rcode = 0;

	now_status();

	return rcode;

cmd_failed:
	cmd_usage(cmdtp);
	return rcode;

}

/* #define U_BOOT_CMD(name,maxargs,rep,cmd,usage,help) */
U_BOOT_CMD(
	voice,	2,	1,	do_voice,
	"Ecocute test command",
	"  control command is 1-11\n"
	"     [ 1] : Speeker On      \n"
	"     [ 2] : Speeker Off     \n"
	"     [ 3] : Microphone On   \n"
	"     [ 4] : Microphone Off  \n"
	"     [ 5] : Volume - Small  \n"
	"     [ 6] : Volume - Middle \n"
	"     [ 7] : Volume - Big    \n"
	"     [ 8] : - not supported \n"
	"     [ 9] : - not supported \n"
	"     [10] : Talk On         \n"
	"     [11] : Talk On         \n"
);


