/*
 * (C) Copyright 2011
 * Working Solar System Production
 *  
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*-----------------------------------------------------------------------
 * Watchdog
 */

#include <common.h>
#include <command.h>

#include <asm/arch/mx25.h>
#include <asm/arch/mx25-regs.h>

#ifdef	CMD_WDOG_DEBUG
# define PRINTF(fmt,args...)	printf (fmt ,##args)
#else
# define PRINTF(fmt,args...)
#endif

extern void wdog_clk_enable(void);
extern void wdog_ping(void);
extern void wdog_config(void);
extern void wdog_set_timeout(u16 timeout);
extern void wdog_enable(void);

/*
 * Watch H/W Testprogram main function
 *
 * wdog command is 
 *  > wdog [reboot time]]
 */
int do_wdog (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int i;
	int rcode = 1;
	u16 wdogtimer;

	if (argc < 2)
		goto cmd_failed;

	wdogtimer = (u16)simple_strtoul(argv[1], NULL, 16) & 0x000000FF;

	printf("watchdog timer is 0x%02x set\n", wdogtimer);

	/* clock enable */
	wdog_clk_enable();

	wdog_config();
	wdog_set_timeout(wdogtimer);
	wdog_enable();
	wdog_ping();

	/* wait system reset */
	for (i = 0; i < (wdogtimer + 4); i++) {
		udelay(500 * 1000);	/* 0.5 sec */
		printf(".");
	}
	printf("system reset ?????\n");

	return 0;

cmd_failed:
	cmd_usage(cmdtp);
	return rcode;
}

/* #define U_BOOT_CMD(name,maxargs,rep,cmd,usage,help) */
U_BOOT_CMD(
	wdog,	2,	1,	do_wdog,
	"Watchdog test command",
	"wdog [time]"
);

int do_mcr (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	u32 mcr;

	if (argc >= 2) {	/* CCM Write */
		mcr = (u32)simple_strtoul(argv[1], NULL, 32);
		*(u32 *)(CCM_MCR) = mcr;
	}
	mcr = *(u32 *)(CCM_MCR);
	printf("MCR Register is %#x\n", mcr);
	printf("  USB XTAL MUX : %#x\n", (mcr & (1 << 31)) >> 31);
	printf("  CLKO EN      : %#x\n", (mcr & (1 << 30)) >> 30);
	printf("  CLKO DIV     : %#x\n", (mcr & (0x3F << 24)) >> 24);
	printf("  CLKO SEL     : %#x\n", (mcr & (0x0F << 20)) >> 20);
	printf("  ESAI CLK MUX : %#x\n", (mcr & (1 << 19)) >> 19);
	printf("  SSI2 CLK MUX : %#x\n", (mcr & (1 << 18)) >> 18);
	printf("  SSI1 CLK MUX : %#x\n", (mcr & (1 << 17)) >> 17);
	printf("  USB CLK      : %#x\n", (mcr & (1 << 16)) >> 16);
	printf("  PER CLK MUX  : %#x\n", (mcr & 0xFFFF));

	return 0;
}

U_BOOT_CMD(
	mcr,	2,	1,	do_mcr,
	"CCM MCR Register Set command",
	"mcr [write data]"
);
