/*
 * (C) Copyright 2009 Freescale Semiconductor
 *
 * (C) Copyright 2004
 * Texas Instruments.
 * Richard Woodruff <r-woodruff2@ti.com>
 * Kshitij Gupta <kshitij@ti.com>
 *
 * Configuration settings for the Freescale i.MX31 PDK board.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H

#include <asm/arch/mx25.h>
#include <asm/arch/mx25-regs.h>

/* High Level Configuration Options */
#define CONFIG_ARM926EJS	1	/* This is an arm1136 CPU core */
#define CONFIG_MX25		1	/* in a mx31 */
#define CONFIG_JHRWL3		1	/* For Sharp Solar System */
#define CONFIG_JHRWL3_PROTO1	1
#define CONFIG_JHRWL3_LCD		/* LCD Setup */

#if defined(CONFIG_JHRWL3)
/* debug board on */
# define CONFIG_JHRWL3_DEBUG

/* H/W Testflag */
# define CONFIG_HW_TESTPROG	1

/* Testprogram 1 releae is Configuration */
/* # define CONFIG_HW_TESTPROG_DENSO	1 */
#endif

#define CONFIG_MX25_HCLK_FREQ	24000000
#define CONFIG_MX25_CLK32	32768

#define CONFIG_IMX_CSPI		1
#define IMX_CSPI_VER_0_7	1

/* IF iMX25 3DS V-1.0 define it */
/* #define CONFIG_MX25_3DS_V10 */

/* IF Sharp Solar 0 Trial production Board defined it */
#define CONFIG_JHRWL3_0TP_BOARD

#if defined(CONFIG_MX25_3DS_V10)
# define MXC_MEMORY_MDDR
#endif

#if defined(CONFIG_JHRWL3_0TP_BOARD)
# define JHRWL3_MEMORY_MDDR
#endif

#define CONFIG_DISPLAY_CPUINFO
#define CONFIG_DISPLAY_BOARDINFO

#define BOARD_LATE_INIT

#define CONFIG_CMDLINE_TAG		1	/* enable passing of ATAGs */
#define CONFIG_SETUP_MEMORY_TAGS	1
#define CONFIG_INITRD_TAG		1

/* No support for NAND boot for i.MX31 PDK yet, so we rely on some other
 * program to initialize the SDRAM.
 */
#define CONFIG_SKIP_RELOCATE_UBOOT

/*
 * Size of malloc() pool
 */
#define CONFIG_SYS_MALLOC_LEN		(CONFIG_ENV_SIZE + 512 * 1024)
#define CONFIG_SYS_GBL_DATA_SIZE	128 /* bytes reserved initial data */

/*
 * Hardware drivers
 */
#define CONFIG_HARD_I2C			1
#define CONFIG_I2C_MXC			1
#define CONFIG_SYS_I2C_PORT		I2C1_BASE
#define CONFIG_SYS_I2C_SPEED		40000
#define CONFIG_SYS_I2C_SLAVE		0xfe

#define CONFIG_MX25_UART		1
#define CONFIG_MX25_UART1		1

/* allow to overwrite serial and ethaddr */
#define CONFIG_ENV_OVERWRITE
#define CONFIG_CONS_INDEX		1
#define CONFIG_BAUDRATE			115200
#define CONFIG_SYS_BAUDRATE_TABLE	{9600, 19200, 38400, 57600, 115200}

/***********************************************************
 * Command definition
 ***********************************************************/

#include <config_cmd_default.h>

#define CONFIG_CMD_I2C
#define CONFIG_CMD_MII
#define CONFIG_CMD_PING
#define CONFIG_CMD_SPI	/* CSPI */
#define CONFIG_CMD_DATE	/* RTC */
#define CONFIG_CMD_NAND
#define CONFIG_CMD_MMC
#define CONFIG_MXC_NAND

#ifdef CONFIG_JHRWL3_0TP_BOARD
/* BootStrap Setting */
/* # define CONFIG_CMD_FUSE */
#endif

#if defined(CONFIG_CMD_DATE)
#define CONFIG_RTC_RX8025
#endif	/* CONFIG_CMD_DATE */

/*
 * MMC Configs
 * */
#if defined(CONFIG_CMD_MMC)
	#define CONFIG_MMC		1
	#define CONFIG_GENERIC_MMC
	#define CONFIG_IMX_MMC
	#define CONFIG_IMX_ESDHC_V1
	#define CONFIG_DOS_PARTITION	1
	#define CONFIG_CMD_FAT		1
#endif

/* Disabled due to compilation errors in cmd_bootm.c (IMLS seems to require
 * that CONFIG_NO_FLASH is undefined).
 */
#undef CONFIG_CMD_IMLS

#define CONFIG_LOADADDR		0x80800000	/* loadaddr env var */

/* Sharp Solar System eth is SMSC */
#if defined(CONFIG_SMC911X)
# undef CONFIG_SMC911X
#endif	/* CONFIG_SMC911X */

#define CONFIG_HAS_ETH1
#define CONFIG_CMD_NET
#define CONFIG_NET_RETRY_COUNT	100
#define CONFIG_CMD_DHCP
#define CONFIG_NET_MULTI
#define CONFIG_ETHPRIME

#define CONFIG_MXC_FEC
#define CONFIG_MII

#define CONFIG_FEC0_IOBASE	FEC_BASE
#define CONFIG_FEC0_PINMUX	-1
#define CONFIG_FEC0_PHY_ADDR	0x0
#define CONFIG_FEC0_MIIBASE	-1

/*
 * Miscellaneous configurable options
 */
#define CONFIG_SYS_LONGHELP		/* undef to save memory */
#define CONFIG_SYS_PROMPT		"jhrwl3 U-Boot > "
#define CONFIG_AUTO_COMPLETE
#define CONFIG_SYS_CBSIZE		256  /* Console I/O Buffer Size */
/* Print Buffer Size */
#define CONFIG_SYS_PBSIZE		(CONFIG_SYS_CBSIZE + sizeof(CONFIG_SYS_PROMPT) + 16)
#define CONFIG_SYS_MAXARGS		16	/* max number of command args */
#define CONFIG_SYS_BARGSIZE		CONFIG_SYS_CBSIZE /* Boot Argument Buf Size */

#define CONFIG_SYS_MEMTEST_START	PHYS_SDRAM_1	/* memtest works on */
#define CONFIG_SYS_MEMTEST_END		0x10000

#define CONFIG_SYS_LOAD_ADDR		CONFIG_LOADADDR /* default load address */

#define CONFIG_SYS_HZ			1000

#define UBOOT_IMAGE_SIZE		0x40000

#define CONFIG_CMDLINE_EDITING		1

/*-----------------------------------------------------------------------
 * Stack sizes
 *
 * The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE	(128 * 1024) /* regular stack */

/*-----------------------------------------------------------------------
 * Physical Memory Map
 */

/* iMX25 V-1.0 has 128MB but V-1.1 has only 64MB */
#if defined(CONFIG_MX25_3DS_V10)
# define CONFIG_NR_DRAM_BANKS	1
# define PHYS_SDRAM_1		CSD0_BASE
# define PHYS_SDRAM_1_SIZE	(128 * 1024 * 1024)
#elif defined(CONFIG_JHRWL3_0TP_BOARD)
/* Sharp Solar is 128MB mDDR * 2 */
# define CONFIG_NR_DRAM_BANKS	2
# define PHYS_SDRAM_1		CSD0_BASE
# define PHYS_SDRAM_2		CSD1_BASE
# define PHYS_SDRAM_1_SIZE	(128 * 1024 * 1024)
# define PHYS_SDRAM_2_SIZE	(128 * 1024 * 1024)
#else
# define CONFIG_NR_DRAM_BANKS	1
# define PHYS_SDRAM_1		CSD0_BASE
# define PHYS_SDRAM_1_SIZE	(64 * 1024 * 1024)
#endif

/* Monitor at beginning of flash */
/* #define CONFIG_FSL_ENV_IN_MMC */
#define CONFIG_FSL_ENV_IN_NAND

/*-----------------------------------------------------------------------
 * FLASH and environment organization
 */


/*
 * NAND Partitions
 *
 * offset	| name		| size		| infomation
 * ======================================================================
 * 0x0000_0000	| bootloader0	| 512K		| BootLoader 0 (U-Boot)
 * 0x0008_0000	| bootloader1	| 512K		| BootLoader 1 (U-Boot)
 * 0x0010_0000	| bootenv0	| 512K		| BootLoader Enviroment 0
 * 0x0018_0000	| bootenv1	| 512K		| BootLoader Enviroment 1
 * 0x0020_0000	| kernel_a	| 8M		| Linux Kernel A side
 * 0x00A0_0000	| kernel_b	| 8M		| Linux Kernel B side
 * 0x0120_0000	| kernel_c	| 8M		| Linux Kernel C side (Reserved)
 * 0x01A0_0000	| kernel_d	| 8M		| Linux Kernel D side (Reserved)
 * 0x0220_0000	| ubi0		| -		| UBIFS
 */
#if defined(CONFIG_JHRWL3_0TP_BOARD)
# define CONFIG_ENV_IS_IN_NAND		1
# define CONFIG_ENV_OFFSET		0x100000
# define CONFIG_ENV_OFFSET_REDUND	0x180000	/* Redund boot enviroment */
# define CONFIG_ENV_SECT_SIZE		(512 * 1024)
# define CONFIG_ENV_SIZE		CONFIG_ENV_SECT_SIZE
#if defined(CONFIG_ENV_OFFSET_REDUND)
# define CONFIG_ENV_SIZE_REDUND		CONFIG_ENV_SIZE
/* enviroment allocate length plus */
# undef CONFIG_SYS_MALLOC_LEN
# define CONFIG_SYS_MALLOC_LEN		((CONFIG_ENV_SIZE * 3) + 512 * 1024)
#endif	/* CONFIG_ENV_OFFSET_REDUND */

#else
#if defined(CONFIG_FSL_ENV_IN_NAND)
# define CONFIG_ENV_IS_IN_NAND		1
# define CONFIG_ENV_OFFSET		0x80000
#elif defined(CONFIG_FSL_ENV_IN_MMC)
# define CONFIG_ENV_IS_IN_MMC		1
# define CONFIG_ENV_OFFSET		(768 * 1024)
#else
# define CONFIG_ENV_IS_NOWHERE		1
#endif

#define CONFIG_ENV_SECT_SIZE		(256 * 1024)
#define CONFIG_ENV_SIZE			CONFIG_ENV_SECT_SIZE
#endif	/* CONFIG_JHRWL3_0TP_BOARD */

#if defined(CONFIG_HW_TESTPROG_DENSO)
#define CONFIG_BOOTDELAY	3
#define CONFIG_ETHADDR		00:04:9F:01:D6:FF
/* ++++++++ Testprogram Demo 1 ++++++++ */
#define CONFIG_EXTRA_ENV_SETTINGS					\
	"netdev=eth0\0"							\
	"kernel=uImage\0"						\
	"bootargs_base=setenv bootargs console=ttymxc0,115200\0"	\
	"bootargs_nfs=setenv bootargs $(bootargs)"			\
		" root=/dev/nfs rw"					\
		" ip=$(ipaddr) nfsroot=$(serverip):$(nfsroot),v3,tcp\0"	\
	"bootargs_nand=setenv bootargs ${bootargs} root=/dev/mtdblock2" \
		" rootfstype=jffs2 ip=dhcp\0"				\
	"bootcmd_nfs=run bootargs_base bootargs_nfs;"			\
		" tftpboot ${loadaddr} ${kernel};bootm\0"		\
	"bootcmd_nand=run bootargs_base bootargs_nand;"			\
		" nand read ${loadaddr} 0x300000 0x500000;bootm\0"	\
	"ecocute_test=uart2test\0"					\
	"bootcmd=run ecocute_test\0"
#else
/* #define CONFIG_WRITE_BOOTLOADER */
#define CONFIG_BOOTDELAY	1
#define CONFIG_ETHADDR		00:04:9F:01:D6:FF
#define CONFIG_SERVERIP		192.168.0.231
#define CONFIG_IPADDR		192.168.0.100
#define CONFIG_NETMASK		255.255.255.0
#define CONFIG_GATEWAYIP	0.0.0.0
#define CONFIG_BOOTARGS		"console=ttymxc0,115200 bootcmd=$(bootmode_args)"
#ifdef CONFIG_WRITE_BOOTLOADER
# define CONFIG_BOOTCOMMAND	"nand erase 0x0 0x40000000;run bootcmd_nfs"
#else
# define CONFIG_BOOTCOMMAND	"flboot"
#endif	/* CONFIG_WRITE_BOOTLOADER */

#define CONFIG_EXTRA_ENV_SETTINGS					\
	"netdev=eth0\0"							\
	"uboot=u-boot.bin\0"						\
	"uboot_addr=0xa0000000\0"					\
	"kernel=uImage\0"						\
	"nfsroot=/nfsroot/rootfs_prod\0"				\
	"ubiroot=ubi0:rootfs_a\0"					\
	"bootmode_args=normal\0"					\
	"bootargs_base=setenv bootargs console=ttymxc0,115200"		\
		" bootcmd=$(bootmode_args)\0"				\
	"bootargs_nfs=setenv bootargs $(bootargs)"			\
		" root=/dev/nfs rw"					\
		" ip=$(ipaddr) nfsroot=$(serverip):$(nfsroot),v3,tcp\0"	\
	"bootcmd_nfs=run bootargs_base bootargs_nfs;"			\
		" tftpboot ${loadaddr} ${kernel};bootm\0"		\
	"bootargs_nand=setenv bootargs ${bootargs} root=$(ubiroot)"	\
		" ubi.mtd=8,1024 rootfstype=ubifs rw\0"			\
	"bootcmd_nand=run bootargs_base bootargs_nand;"			\
		" nand read ${loadaddr} 0x200000 0x800000;bootm\0"
#endif	/* CONFIG_HW_TESTPROG_DENSO */


/*-----------------------------------------------------------------------
 * NAND FLASH driver setup
 */
#define CONFIG_SYS_NO_FLASH
#define NAND_MAX_CHIPS			1
#define CONFIG_SYS_MAX_NAND_DEVICE	1
#define CONFIG_SYS_NAND_BASE		0x40000000

/*
 * JFFS2 partitions TODO:
 */
#undef CONFIG_JFFS2_CMDLINE
#define CONFIG_JFFS2_DEV		"nand0"




/*-----------------------------------------------------------------------
 * H/W Test Program Configuration
 *  - CPU Core     : none
 *  - SDRAM        : mtest
 *  - NAND_Flash   : Nand Command
 *  - UART1        : none
 *  - UART2        : uart2test
 *  - UART3        : uart3test
 *  - Ethernet     : mii / ping
 *  - RTC          : date
 *  - EEPROM       : eeprom
 *  - Switch       : switchcheck
 *  - LED          : led
 *  - VDD          : none
 *  - Ecocute      : voice
 *  - Speech       : speech
 *  - Watchdog     : wdog
 */
#ifdef CONFIG_HW_TESTPROG

/*-----------------------------------------------------------------------
 * CPU Core
 */
// NONE //

/*-----------------------------------------------------------------------
 * SDRAM
 */
#ifndef CONFIG_SYS_DRAM_TEST
# define CONFIG_SYS_DRAM_TEST
#endif	/* CONFIG_SYS_DRAM_TEST */

/* mtest */
#ifndef CONFIG_CMD_MEMORY
# define CONFIG_CMD_MEMORY	/* cmd_mem.c */
#endif

#ifdef CONFIG_SYS_MEMTEST_START
# undef CONFIG_SYS_MEMTEST_START
#endif
#ifdef CONFIG_SYS_MEMTEST_END
# undef CONFIG_SYS_MEMTEST_END
#endif

/* mtest area */
#define CONFIG_SYS_MEMTEST_START	CSD0_BASE
#define CONFIG_SYS_MEMTEST_END		0x08000000


/*-----------------------------------------------------------------------
 * NAND-Flash
 */
/* nand */
#ifndef CONFIG_CMD_NAND
# define CONFIG_CMD_NAND		/* cmd_nand.c */
#endif


/*-----------------------------------------------------------------------
 * UART1
 */
// NONE //


/*-----------------------------------------------------------------------
 * UART2
 */
/* uart2test */
#ifndef CONFIG_CMD_UART2_TEST
# define CONFIG_CMD_UART2_TEST	/* add : cmd_uart.c */
#endif


/*-----------------------------------------------------------------------
 * UART3
 */
/* uart3test */
#ifndef CONFIG_CMD_UART3_TEST
# define CONFIG_CMD_UART3_TEST	/* add : cmd_uart.c */
#endif


/*-----------------------------------------------------------------------
 * Ethernet
 */
/* mii */
#ifndef CONFIG_CMD_MII
# define CONFIG_CMD_MII		/* cmd_mii.c */
#endif

/*
#ifndef CONFIG_MII_INIT
#define CONFIG_MII_INIT
#endif
*/

/* ping */
#ifndef CONFIG_CMD_PING
# define CONFIG_CMD_PING
#endif


/*-----------------------------------------------------------------------
 * RTC
 */
/* date */
#ifndef CONFIG_CMD_DATE
# define CONFIG_CMD_DATE		/* cmd_date.c */
#endif

#ifdef CONFIG_SYS_I2C_RTC_ADDR
# undef CONFIG_SYS_I2C_RTC_ADDR
#endif

#ifdef CONFIG_JHRWL3_PROTO1
# ifndef CONFIG_RTC_PCF8563
#  define CONFIG_RTC_PCF8563	/* driver/pcf8563.c */
#  define CONFIG_SYS_I2C_RTC_ADDR	(0x51)
# endif
#else
# ifndef CONFIG_RTC_RX8025
#  define CONFIG_RTC_RX8025	/* driver/rx8025.c */
#  define CONFIG_SYS_I2C_RTC_ADDR	(0x32)
# endif
#endif	/* CONFIG_JHRWL3_PROTO1 */

/*-----------------------------------------------------------------------
 * EEPROM
 */
/* eeprom */
#ifndef CONFIG_CMD_EEPROM
# define CONFIG_CMD_EEPROM
#endif

#ifndef CONFIG_SYS_EEPROM_WREN
# define CONFIG_SYS_EEPROM_WREN	/* Write Protect */
#endif

#ifdef CONFIG_SYS_I2C_EEPROM_ADDR
# undef CONFIG_SYS_I2C_EEPROM_ADDR
#endif
#define CONFIG_SYS_I2C_EEPROM_ADDR 0x50

#ifndef CONFIG_SYS_I2C_EEPROM_ADDR_LEN
# define CONFIG_SYS_I2C_EEPROM_ADDR_LEN 1
#endif

/*-----------------------------------------------------------------------
 * SWITCH
 */
/* switchcheck */
#ifndef CONFIG_CMD_SWITCH
# define CONFIG_CMD_SWITCH	/* add : cmd_switch.c */
#endif


/*-----------------------------------------------------------------------
 * LED
 */
/* led */
#ifndef CONFIG_CMD_LED
# define CONFIG_CMD_LED		/* add : cmd_led.c */
#endif


/*-----------------------------------------------------------------------
 * VDD
 */
// NONE //


/*-----------------------------------------------------------------------
 * Ecocute
 */
/* voice */
#ifndef CONFIG_CMD_VOICE
# define CONFIG_CMD_VOICE	/* add : cmd_voice.c */
#endif


/*-----------------------------------------------------------------------
 * Speech
 */
/* speech */
#ifndef CONFIG_CMD_SPEECH
# define CONFIG_CMD_SPEECH	/* add : cmd_speech.c */
#endif

#define CONFIG_SYS_SPI_SPEECH_DEVID	0

/*-----------------------------------------------------------------------
 * Watchdog
 */
/* wdog */
#ifndef CONFIG_CMD_WDOG
# define CONFIG_CMD_WDOG		/* add : cmd_wdog.c */
#endif


#endif	/* CONFIG_HW_TESTPROG */

#endif /* __CONFIG_H */


